@@ -1,21 +1,20 @@
---[[
+--[[ $%BEGINLICENSE%$
+ Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
 
-   Copyright (C) 2007 MySQL AB
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
---]]
+ $%ENDLICENSE%$ --]]
 
 local commands     = require("proxy.commands")
 local tokenizer    = require("proxy.tokenizer")
@@ -42,17 +41,12 @@ end
 -- @param inj "packet.injection"
 function resultset_to_string( res )
 	local s = ""
-	-- TODO - check boundaries. Probably buggy after changes in rev 262
-	local field_count = 0      
 	local fields = res.fields
 
 	if not fields then return "" end
-	print (fields[field_count])	
-	while fields[field_count] do
-		local field = fields[field_count]
-		field_count = field_count + 1
-	end
-	
+
+	local field_count = #fields
+
 	local row_count = 0
 	for row in res.rows do
 		local o
@@ -68,13 +62,13 @@ function resultset_to_string( res )
 			if not row[i] then
 				o = o .. "NULL"
 			-- TODO check if the indexes are in range 
-			elseif fields[i - 1].type == proxy.MYSQL_TYPE_STRING or  
-				fields[i - 1].type == proxy.MYSQL_TYPE_VAR_STRING or 
-				fields[i - 1].type == proxy.MYSQL_TYPE_LONG_BLOB or 
-				fields[i - 1].type == proxy.MYSQL_TYPE_MEDIUM_BLOB then
+			elseif fields[i].type == proxy.MYSQL_TYPE_STRING or  
+				fields[i].type == proxy.MYSQL_TYPE_VAR_STRING or 
+				fields[i].type == proxy.MYSQL_TYPE_LONG_BLOB or 
+				fields[i].type == proxy.MYSQL_TYPE_MEDIUM_BLOB then
 				o = o .. string.format("%q", row[i])
 			else
-				-- print("  [".. i .."] field-type: " .. fields[i - 1].type)
+				-- print("  [".. i .."] field-type: " .. fields[i].type)
 				o = o .. row[i]
 			end
 		end
@@ -87,7 +81,7 @@ end
 
 
 function math.rolling_avg_init()
-	return { count = 0, value = 0 }
+	return { count = 0, value = 0, sum_value = 0 }
 end
 
 function math.rolling_stddev_init()
@@ -96,10 +90,9 @@ end
 
 function math.rolling_avg(val, tbl) 
 	tbl.count = tbl.count + 1
+	tbl.sum_value = tbl.sum_value + val
 
-	-- scale the old avg to the full-sum
-	-- add the current val and divide it against the new count
-	tbl.value = ((tbl.value * (tbl.count - 1)) + val) / tbl.count
+	tbl.value = tbl.sum_value / tbl.count
 
 	return tbl.value
 end
@@ -150,6 +143,14 @@ function read_query(packet)
 	tokens = tokenizer.tokenize(cmd.query)
 	norm_query = tokenizer.normalize(tokens)
 
+	-- create a id for this query
+	query_id   = ("%s.%s.%s"):format(
+		proxy.connection.backend_ndx, 
+		proxy.connection.client.default_db ~= "" and 
+			proxy.connection.client.default_db or 
+			"(null)", 
+		norm_query)
+
 	-- handle the internal data
 	if norm_query == "SELECT * FROM `histogram` . `queries` " then
 		proxy.response = {
@@ -173,7 +174,7 @@ function read_query(packet)
 			local cutoff = proxy.global.config.analyze_query.query_cutoff
 
 			for k, v in pairs(proxy.global.queries) do
-				local q = k
+				local q = v.query
 
 				if cutoff and cutoff < #k then
 					q = k:sub(1, cutoff) .. "..."
@@ -209,24 +210,25 @@ function read_query(packet)
 	--
 	local queries = proxy.global.queries
 
-	if not queries[norm_query] then
-		queries[norm_query] = {
-			query = norm_query,
-			avg = math.rolling_avg_init(),
+	if not queries[query_id] then
+		queries[query_id] = {
+			db     = proxy.connection.client.default_db,
+			query  = norm_query,
+			avg    = math.rolling_avg_init(),
 			stddev = math.rolling_stddev_init()
 		}
 	end
 
 
-	if queries[norm_query].do_analyze then
+	if queries[query_id].do_analyze then
 		-- wrap the query in SHOW SESSION STATUS
 		--
 		-- note: not used yet
-		proxy.queries:append(2, string.char(proxy.COM_QUERY) .. "SHOW SESSION STATUS")
-		proxy.queries:append(1, packet)
-		proxy.queries:append(3, string.char(proxy.COM_QUERY) .. "SHOW SESSION STATUS")
+		proxy.queries:append(2, string.char(proxy.COM_QUERY) .. "SHOW SESSION STATUS", { resultset_is_needed = true })
+		proxy.queries:append(1, packet, { resultset_is_needed = true }) -- FIXME: this should work without it being set (inj.resultset.query_status)
+		proxy.queries:append(3, string.char(proxy.COM_QUERY) .. "SHOW SESSION STATUS", { resultset_is_needed = true })
 	else
-		proxy.queries:append(1, packet)
+		proxy.queries:append(1, packet, { resultset_is_needed = true }) -- FIXME: this one too
 	end
 
 	return proxy.PROXY_SEND_QUERY
@@ -269,7 +271,7 @@ function read_query_result(inj)
 		local avg_query_time    = math.rolling_avg(   inj.query_time, bl.avg)
 		local stddev_query_time = math.rolling_stddev(inj.query_time, bl.stddev)
 
-		local st = proxy.global.queries[norm_query]
+		local st = proxy.global.queries[query_id]
 		local q_avg_query_time    = math.rolling_avg(   inj.query_time, st.avg)
 		local q_stddev_query_time = math.rolling_stddev(inj.query_time, st.stddev)
 
@@ -307,12 +309,14 @@ function read_query_result(inj)
 		end
 
 		if log_query and config.auto_processlist then
-			proxy.queries:append(4, string.char(proxy.COM_QUERY) .. "SHOW FULL PROCESSLIST")
+			proxy.queries:append(4, string.char(proxy.COM_QUERY) .. "SHOW FULL PROCESSLIST",
+				{ resultset_is_needed = true })
 		end
 	
 		if log_query and config.auto_explain then
 			if tokens[1] and tokens[1].token_name == "TK_SQL_SELECT" then
-				proxy.queries:append(5, string.char(proxy.COM_QUERY) .. "EXPLAIN " .. inj.query:sub(2))
+				proxy.queries:append(5, string.char(proxy.COM_QUERY) .. "EXPLAIN " .. inj.query:sub(2),
+					{ resultset_is_needed = true })
 			end
 		end
 
