@@ -1,21 +1,20 @@
---[[
+--[[ $%BEGINLICENSE%$
+ Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
 
-   Copyright (C) 2007 MySQL AB
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
---]]
+ $%ENDLICENSE%$ --]]
 
 module("proxy.commands", package.seeall)
 
@@ -68,18 +67,25 @@ function parse(packet)
 	
 	if cmd.type == proxy.COM_QUERY then
 		cmd.query = packet:sub(2)
-	elseif cmd.type == proxy.COM_INIT_DB then
-		cmd.schema = packet:sub(2)
-	elseif cmd.type == proxy.COM_QUIT then
-	elseif cmd.type == proxy.COM_PING then
-	elseif cmd.type == proxy.COM_SHUTDOWN then
+	elseif cmd.type == proxy.COM_QUIT or
+	       cmd.type == proxy.COM_PING or
+	       cmd.type == proxy.COM_SHUTDOWN then
 		-- nothing to decode
 	elseif cmd.type == proxy.COM_STMT_PREPARE then
 		cmd.query = packet:sub(2)
+	-- the stmt_handler_id is at the same position for both STMT_EXECUTE and STMT_CLOSE
+	elseif cmd.type == proxy.COM_STMT_EXECUTE or cmd.type == proxy.COM_STMT_CLOSE then
+		cmd.stmt_handler_id = string.byte(packet, 2) + (string.byte(packet, 3) * 256) + (string.byte(packet, 4) * 256 * 256) + (string.byte(packet, 5) * 256 * 256 * 256)
 	elseif cmd.type == proxy.COM_FIELD_LIST then
-		-- should have a table-name
+		cmd.table = packet:sub(2)
+	elseif cmd.type == proxy.COM_INIT_DB or
+	       cmd.type == proxy.COM_CREATE_DB or
+	       cmd.type == proxy.COM_DROP_DB then
+		cmd.schema = packet:sub(2)
+	elseif cmd.type == proxy.COM_SET_OPTION then
+		cmd.option = packet:sub(2)
 	else
-		print("[debug] (command) unhandled type " .. cmd.type_name)
+		print("[debug] (command) unhandled type name:" .. tostring(cmd.type_name) .. " byte:" .. tostring(cmd.type))
 	end
 
 	return cmd
@@ -98,6 +104,10 @@ function pretty_print(cmd)
 	elseif cmd.type == proxy.COM_FIELD_LIST then
 		-- should have a table-name
 		return ("[%s]"):format(cmd.type_name)
+	elseif cmd.type == proxy.COM_STMT_EXECUTE then
+    	return ("[%s] %s"):format(cmd.type_name, cmd.stmt_handler_id)
+	elseif cmd.type == proxy.COM_STMT_EXECUTE then
+		return ("[%s] %s"):format(cmd.type_name, cmd.stmt_handler_id)
 	end
 
 	return ("[%s] ... no idea"):format(cmd.type_name)
