@@ -15,7 +15,66 @@
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
  $%ENDLICENSE%$ */
- 
+
+/**
+ * @page page-plugin-admin Administration plugin
+ *
+ * The admin plugin exposes the internals of the MySQL Proxy on a SQL interface 
+ * to the outside world. 
+ *
+ * @section plugin-admin-options Configuration
+ *
+ * @li @c --admin-address    defaults to @c :4041
+ * @li @c --admin-lua-script specifies the lua script to load that exposes handles the SQL statements
+ * @li @c --admin-username   username
+ * @li @c --admin-password   password
+ *
+ * @section plugin-admin-implementation Implementation
+ *
+ * The admin plugin handles two SQL queries by default that are used by the mysql commandline client when
+ * it logins to expose the version string and username. All other queries are returned with an error if they 
+ * are not handled by the Lua script (@c --admin-lua-script). 
+ *
+ * The script provides a @c read_query() function which returns a result-set in the same way as the proxy
+ * module does:
+ *
+ * @include lib/admin.lua
+ *
+ * @section plugin-admin-missing To fix before 1.0
+ *
+ * Before MySQL Proxy 1.0 we have to cleanup the admin plugin to:
+ *
+ * @li replace the hard-coded username, password by a real credential store @see network_mysqld_admin_plugin_apply_config()
+ * @li provide a full fleged admin script that exposes all the internal stats @see lib/admin.lua
+ *
+ * @section plugin-admin-backends Backends 
+ *
+ * @b TODO The admin plugin should also be able to change and add the information about the backends
+ * while the MySQL Proxy is running. It is stored in the @c proxy.global.backends table can be mapped to SQL commands.
+ *
+ * @li support for @c SHOW @c CREATE @c TABLE should return @code
+ *   CREATE TABLE backends {
+ *     id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
+ *     address VARCHAR(...) NOT NULL,
+ *     port INT,
+ *     is_enabled INT NOT NULL, -- 0 or 1, a bool
+ *   }
+ * @endcode
+ * @li getting all backends @code
+ *   SELECT * FROM backends;
+ *   SELECT * FROM backends WHERE id = 1;
+ * @endcode
+ * @li disable backends (a flag needs to be added to the backend code) @code
+ *   UPDATE backends SET is_enabled = 0;
+ * @endcode
+ * @li adding and removing backends like @code
+ *   INSERT INTO backends ( address, port ) VALUES ( "10.0.0.20", 3306 );
+ *   DELETE backends WHERE id = 1;
+ * @endcode
+ *
+ * In a similar way the @c config section of @c proxy.global should be exposed allowing the admin plugin to change the
+ * configuration at runtime. @see lib/proxy/auto-config.lua
+ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -34,29 +93,6 @@
 
 #include <gmodule.h>
 
-/**
- * a simple query handler
- *
- * we handle the basic statements that the mysql-client sends us
- *
- * @todo we have to split the queries into a basic SQL syntax:
- *
- *   SELECT * 
- *     FROM table
- *   [WHERE field = value]
- *
- * - no joins
- * - no grouping
- *
- *   DELETE
- *     FROM table
- * 
- * - no WHERE clause
- *
- * - each table has to have a provider for a table 
- * - each plugin should able to provide tables as needed
- */
-
 #define C(x) x, sizeof(x) -1
 #define S(x) x->str, x->len
 
@@ -99,7 +135,6 @@ int network_mysqld_con_handle_stmt(chassis G_GNUC_UNUSED *chas, network_mysqld_c
 			g_ptr_array_add(row, g_strdup("MySQL Enterprise Agent"));
 			g_ptr_array_add(rows, row);
 
-			con->client->packet_id++;
 			network_mysqld_con_send_resultset(con->client, fields, rows);
 			
 		} else if (0 == g_ascii_strncasecmp(s->str + NET_HEADER_SIZE + 1, C("select USER()"))) {
@@ -116,10 +151,8 @@ int network_mysqld_con_handle_stmt(chassis G_GNUC_UNUSED *chas, network_mysqld_c
 			g_ptr_array_add(row, g_strdup("root"));
 			g_ptr_array_add(rows, row);
 
-			con->client->packet_id++;
 			network_mysqld_con_send_resultset(con->client, fields, rows);
 		} else {
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("(admin-server) query not known"));
 		}
 
@@ -147,11 +180,9 @@ int network_mysqld_con_handle_stmt(chassis G_GNUC_UNUSED *chas, network_mysqld_c
 	case COM_QUIT:
 		break;
 	case COM_INIT_DB:
-		con->client->packet_id++;
 		network_mysqld_con_send_ok(con->client);
 		break;
 	default:
-		con->client->packet_id++;
 		network_mysqld_con_send_error(con->client, C("unknown COM_*"));
 		break;
 	}
@@ -177,7 +208,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_con_init) {
 	network_mysqld_proto_append_auth_challenge(packet, challenge);
 	con->client->challenge = challenge;
 
-	network_mysqld_queue_append(con->client->send_queue, S(packet), 0);
+	network_mysqld_queue_append(con->client, con->client->send_queue, S(packet));
 
 	g_string_free(packet, TRUE);
 	
@@ -195,17 +226,14 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_auth) {
 	network_socket *recv_sock, *send_sock;
 	network_mysqld_auth_response *auth;
 	GString *excepted_response;
+	GString *hashed_password;
 	
 	recv_sock = con->client;
 	send_sock = con->client;
 
-	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
+	packet.data = g_queue_peek_head(recv_sock->recv_queue->chunks);
 	packet.offset = 0;
 
-	if (packet.data->len != recv_sock->packet_len + NET_HEADER_SIZE) {
-		return NETWORK_SOCKET_SUCCESS; /* we are not finished yet */
-	}
-
 	/* decode the packet */
 	network_mysqld_proto_skip_network_header(&packet);
 
@@ -216,16 +244,18 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_auth) {
 	
 	con->client->response = auth;
 	
-	send_sock->packet_id = recv_sock->packet_id + 1;
-
 	/* check if the password matches */
 	excepted_response = g_string_new(NULL);
+	hashed_password = g_string_new(NULL);
 
 	if (!strleq(S(con->client->response->username), con->config->admin_username, strlen(con->config->admin_username))) {
 		network_mysqld_con_send_error_full(send_sock, C("unknown user"), 1045, "28000");
 		
 		con->state = CON_STATE_SEND_ERROR; /* close the connection after we have sent this packet */
-	} else if (network_mysqld_proto_scramble(excepted_response, recv_sock->challenge->challenge, con->config->admin_password)) {
+	} else if (network_mysqld_proto_password_hash(hashed_password, con->config->admin_password, strlen(con->config->admin_password))) {
+	} else if (network_mysqld_proto_password_scramble(excepted_response,
+				S(recv_sock->challenge->challenge),
+				S(hashed_password))) {
 		network_mysqld_con_send_error_full(send_sock, C("scrambling failed"), 1045, "28000");
 		
 		con->state = CON_STATE_SEND_ERROR; /* close the connection after we have sent this packet */
@@ -238,10 +268,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_auth) {
 	
 		con->state = CON_STATE_SEND_AUTH_RESULT;
 	}
-	
+
+	g_string_free(hashed_password, TRUE);	
 	g_string_free(excepted_response, TRUE);
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
 	g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 	
 
@@ -268,9 +298,6 @@ static network_mysqld_lua_stmt_ret admin_lua_read_query(network_mysqld_con *con)
 		    0 == g_ascii_strncasecmp(packet->str + NET_HEADER_SIZE + 1, C("LOAD "))) return PROXY_SEND_QUERY;
 	}
 
-	/* reset the query status */
-	memset(&(st->injected.qstat), 0, sizeof(st->injected.qstat));
-	
 	network_injection_queue_reset(st->injected.queries);
 
 	/* ok, here we go */
@@ -280,12 +307,10 @@ static network_mysqld_lua_stmt_ret admin_lua_read_query(network_mysqld_con *con)
 		case REGISTER_CALLBACK_SUCCESS:
 			break;
 		case REGISTER_CALLBACK_LOAD_FAILED:
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to load. Check the error log."));
 			con->state = CON_STATE_SEND_ERROR;
 			return PROXY_SEND_RESULT;
 		case REGISTER_CALLBACK_EXECUTE_FAILED:
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to execute. Check the error log."));
 			con->state = CON_STATE_SEND_ERROR;
 			return PROXY_SEND_RESULT;
@@ -341,8 +366,6 @@ static network_mysqld_lua_stmt_ret admin_lua_read_query(network_mysqld_con *con)
 				 *
 				 */
 	
-				con->client->packet_id++;
-
 				if (network_mysqld_con_lua_handle_proxy_response(con, con->config->lua_script)) {
 					/**
 					 * handling proxy.response failed
@@ -415,15 +438,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_query) {
 	
 	packet = chunk->data;
 
-	if (packet->len != recv_sock->packet_len + NET_HEADER_SIZE) return NETWORK_SOCKET_SUCCESS;
-
-	con->parse.len = recv_sock->packet_len;
-
 	ret = admin_lua_read_query(con);
 
 	switch (ret) {
 	case PROXY_NO_DECISION:
-		con->client->packet_id++;
 		network_mysqld_con_send_error(con->client, C("need a resultset + proxy.PROXY_SEND_RESULT"));
 		con->state = CON_STATE_SEND_ERROR;
 		break;
@@ -431,14 +449,12 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_query) {
 		con->state = CON_STATE_SEND_QUERY_RESULT;
 		break; 
 	default:
-		con->client->packet_id++;
 		network_mysqld_con_send_error(con->client, C("need a resultset + proxy.PROXY_SEND_RESULT ... got something else"));
 
 		con->state = CON_STATE_SEND_ERROR;
 		break;
 	}
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
 	g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
 	return NETWORK_SOCKET_SUCCESS;
