@@ -22,7 +22,7 @@
 #endif
 
 /** 
- * @page proxy_states The internal states of the Proxy
+ * @page page-plugin-proxy Proxy plugin
  *
  * The MySQL Proxy implements the MySQL Protocol in its own way. 
  *
@@ -144,6 +144,8 @@ typedef int socklen_t;
 
 #include "lua-load-factory.h"
 
+#include "chassis-timings.h"
+
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
 
@@ -186,6 +188,7 @@ struct chassis_plugin_config {
 
 static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_con *con) {
 	network_socket *send_sock = con->client;
+	network_socket *recv_sock = con->server;
 	injection *inj = NULL;
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
 	network_mysqld_lua_stmt_ret ret = PROXY_NO_DECISION;
@@ -207,12 +210,10 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_co
 		case REGISTER_CALLBACK_SUCCESS:
 			break;
 		case REGISTER_CALLBACK_LOAD_FAILED:
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to load. Check the error log."));
 			con->state = CON_STATE_SEND_ERROR;
 			return PROXY_SEND_RESULT;
 		case REGISTER_CALLBACK_EXECUTE_FAILED:
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to execute. Check the error log."));
 			con->state = CON_STATE_SEND_ERROR;
 			return PROXY_SEND_RESULT;
@@ -234,7 +235,7 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_co
 			inj_p = lua_newuserdata(L, sizeof(inj));
 			*inj_p = inj;
 
-			inj->result_queue = con->client->send_queue->chunks;
+			inj->result_queue = con->server->recv_queue->chunks;
 
 			proxy_getinjectionmetatable(L);
 			lua_setmetatable(L, -2);
@@ -267,13 +268,11 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_co
 				/**
 				 * replace the result-set the server sent us 
 				 */
-				while ((packet = g_queue_pop_head(send_sock->send_queue->chunks))) g_string_free(packet, TRUE);
+				while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(packet, TRUE);
 				
 				/**
 				 * we are a response to the client packet, hence one packet id more 
 				 */
-				send_sock->packet_id++;
-
 				if (network_mysqld_con_lua_handle_proxy_response(con, con->config->lua_script)) {
 					/**
 					 * handling proxy.response failed
@@ -292,6 +291,9 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_co
 					/**
 					 * make sure we send only one result-set per client-query
 					 */
+					while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) {
+						network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, packet);
+					}
 					st->injected.sent_resultset++;
 					break;
 				}
@@ -320,7 +322,7 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_co
 					break;
 				}
 
-				while ((packet = g_queue_pop_head(send_sock->send_queue->chunks))) g_string_free(packet, TRUE);
+				while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(packet, TRUE);
 
 				break;
 			default:
@@ -468,17 +470,14 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_handshake) {
 
 	/* handle ERR packets directly */
 	if (status == 0xff) {
-		send_sock->packet_id = recv_sock->packet_id;
-
 		/* move the chunk from one queue to the next */
-		network_queue_append(send_sock->send_queue, g_queue_pop_tail(recv_sock->recv_queue->chunks));
+		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, g_queue_pop_tail(recv_sock->recv_queue->chunks));
 
 		return NETWORK_SOCKET_ERROR; /* it sends what is in the send-queue and hangs up */
 	}
 
 	challenge = network_mysqld_auth_challenge_new();
 	if (network_mysqld_proto_get_auth_challenge(&packet, challenge)) {
-		recv_sock->packet_len = PACKET_LEN_UNSET;
  		g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
 		network_mysqld_auth_challenge_free(challenge);
@@ -499,7 +498,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_handshake) {
 		/* the client overwrote and wants to send its own packet
 		 * it is already in the queue */
 
-		recv_sock->packet_len = PACKET_LEN_UNSET;
  		g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
 		return NETWORK_SOCKET_ERROR;
@@ -508,13 +506,13 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_handshake) {
 		break;
 	} 
 
-	challenge_packet = g_string_new(NULL);
+	challenge_packet = g_string_sized_new(packet.data->len); /* the packet we generate will be likely as large as the old one. should save some reallocs */
 	network_mysqld_proto_append_auth_challenge(challenge_packet, challenge);
-	network_mysqld_queue_append(send_sock->send_queue, S(challenge_packet), recv_sock->packet_id);
+	network_mysqld_queue_sync(send_sock, recv_sock);
+	network_mysqld_queue_append(send_sock, send_sock->send_queue, S(challenge_packet));
 
 	g_string_free(challenge_packet, TRUE);
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
 	g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
 	/* copy the pack to the client */
@@ -571,8 +569,6 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_auth(network_mysqld_con *con)
 		case PROXY_SEND_RESULT:
 			/* answer directly */
 
-			con->client->packet_id++;
-			
 			if (network_mysqld_con_lua_handle_proxy_response(con, con->config->lua_script)) {
 				/**
 				 * handling proxy.response failed
@@ -583,6 +579,16 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_auth(network_mysqld_con *con)
 				network_mysqld_con_send_error(con->client, C("(lua) handling proxy.response failed, check error-log"));
 			}
 
+			break;
+		case PROXY_SEND_QUERY:
+			/* something is in the injection queue, pull it from there and replace the content of
+			 * original packet */
+
+			if (st->injected.queries->length) {
+				ret = PROXY_SEND_INJECTION;
+			} else {
+				ret = PROXY_NO_DECISION;
+			}
 			break;
 		default:
 			ret = PROXY_NO_DECISION;
@@ -612,6 +618,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 	network_mysqld_auth_response *auth;
 	int err = 0;
 	gboolean free_client_packet = TRUE;
+	network_mysqld_con_lua_t *st = con->plugin_con_state;
 
 	recv_sock = con->client;
 	send_sock = con->server;
@@ -619,8 +626,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
  	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
 	packet.offset = 0;
 
-	if (packet.data->len != recv_sock->packet_len + NET_HEADER_SIZE) return NETWORK_SOCKET_SUCCESS; /* we are not finished yet */
-
 	err = err || network_mysqld_proto_skip_network_header(&packet);
 	if (err) return NETWORK_SOCKET_ERROR;
 
@@ -633,7 +638,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 		return NETWORK_SOCKET_ERROR;
 	}
 	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
-		network_mysqld_queue_append(con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"), con->client->packet_id + 1);
+		/* should use packet-id 0 */
+		network_mysqld_queue_append(con->client, con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"));
 		return NETWORK_SOCKET_ERROR;
 	}
 
@@ -649,26 +655,26 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 		con->state = CON_STATE_SEND_AUTH_RESULT;
 
 		break;
+	case PROXY_SEND_INJECTION: {
+		injection *inj;
+
+		/* replace the client challenge that is sent to the server */
+		inj = g_queue_pop_head(st->injected.queries);
+
+		network_mysqld_queue_append(send_sock, send_sock->send_queue, S(inj->query));
+
+		injection_free(inj);
+
+		con->state = CON_STATE_SEND_AUTH;
+
+		break; }
 	case PROXY_NO_DECISION:
 		/* if we don't have a backend (con->server), we just ack the client auth
 		 */
 		if (!con->server) {
-			GString *auth_resp_packet;
-			network_mysqld_ok_packet_t *ok_packet;
-
 			con->state = CON_STATE_SEND_AUTH_RESULT;
-		
-			auth_resp_packet = g_string_new(NULL);
-			ok_packet = network_mysqld_ok_packet_new();
-			ok_packet->server_status = SERVER_STATUS_AUTOCOMMIT;
 
-			network_mysqld_proto_append_ok_packet(auth_resp_packet, ok_packet);
-
-			network_mysqld_queue_append(recv_sock->send_queue, 
-					S(auth_resp_packet), 2);
-
-			network_mysqld_ok_packet_free(ok_packet);
-			g_string_free(auth_resp_packet, TRUE);
+			network_mysqld_con_send_ok(recv_sock);
 
 			break;
 		}
@@ -682,6 +688,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 		if (con->server->is_authed) {
 			if (config->pool_change_user) {
 				GString *com_change_user = g_string_new(NULL);
+
 				/* copy incl. the nul */
 				g_string_append_c(com_change_user, COM_CHANGE_USER);
 				g_string_append_len(com_change_user, con->client->response->username->str, con->client->response->username->len + 1); /* nul-term */
@@ -693,9 +700,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 
 				g_string_append_len(com_change_user, con->client->default_db->str, con->client->default_db->len + 1);
 				
-				network_mysqld_queue_append(send_sock->send_queue, 
-						S(com_change_user), 
-						0);
+				network_mysqld_queue_append(
+						send_sock,
+						send_sock->send_queue, 
+						S(com_change_user));
 
 				/**
 				 * the server is already authenticated, the client isn't
@@ -708,6 +716,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 				con->state = CON_STATE_SEND_AUTH;
 			} else {
 				GString *auth_resp;
+
 				/* check if the username and client-scramble are the same as in the previous authed
 				 * connection */
 
@@ -738,14 +747,13 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 					network_mysqld_ok_packet_free(ok_packet);
 				}
 
-				network_mysqld_queue_append(recv_sock->send_queue, 
-						S(auth_resp), 
-						2);
+				network_mysqld_queue_append(recv_sock, recv_sock->send_queue, 
+						S(auth_resp));
 
 				g_string_free(auth_resp, TRUE);
 			}
 		} else {
-			network_queue_append(send_sock->send_queue, packet.data);
+			network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, packet.data);
 			con->state = CON_STATE_SEND_AUTH;
 
 			free_client_packet = FALSE; /* the packet.data is now part of the send-queue, don't free it further down */
@@ -757,7 +765,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 		break;
 	}
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
 	if (free_client_packet) {
 		g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 	} else {
@@ -867,9 +874,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth_result) {
 	chunk = recv_sock->recv_queue->chunks->tail;
 	packet = chunk->data;
 
-	/* we aren't finished yet */
-	if (packet->len != recv_sock->packet_len + NET_HEADER_SIZE) return NETWORK_SOCKET_SUCCESS;
-
 	/* send the auth result to the client */
 	if (con->server->is_authed) {
 		/**
@@ -910,7 +914,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth_result) {
 		
 		break;
 	case PROXY_NO_DECISION:
-		network_queue_append(send_sock->send_queue, packet);
+		network_mysqld_queue_append_raw(
+				send_sock,
+				send_sock->send_queue,
+				packet);
 
 		break;
 	default:
@@ -921,9 +928,15 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth_result) {
 	/**
 	 * we handled the packet on the server side, free it
 	 */
-	recv_sock->packet_len = PACKET_LEN_UNSET;
 	g_queue_delete_link(recv_sock->recv_queue->chunks, chunk);
 	
+	/* the auth phase is over
+	 *
+	 * reset the packet-id sequence
+	 */
+	network_mysqld_queue_reset(send_sock);
+	network_mysqld_queue_reset(recv_sock);
+	
 	con->state = CON_STATE_SEND_AUTH_RESULT;
 
 	return NETWORK_SOCKET_SUCCESS;
@@ -961,9 +974,6 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
 	
 	}
 
-	/* reset the query status */
-	memset(&(st->injected.qstat), 0, sizeof(st->injected.qstat));
-	
 	network_injection_queue_reset(st->injected.queries);
 
 	/* ok, here we go */
@@ -973,12 +983,10 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
 		case REGISTER_CALLBACK_SUCCESS:
 			break;
 		case REGISTER_CALLBACK_LOAD_FAILED:
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to load. Check the error log."));
 			con->state = CON_STATE_SEND_ERROR;
 			return PROXY_SEND_RESULT;
 		case REGISTER_CALLBACK_EXECUTE_FAILED:
-			con->client->packet_id++;
 			network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to execute. Check the error log."));
 			con->state = CON_STATE_SEND_ERROR;
 			return PROXY_SEND_RESULT;
@@ -1008,9 +1016,16 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
 		 */
 		lua_getfield_literal(L, -1, C("read_query"));
 		if (lua_isfunction(L, -1)) {
+			luaL_Buffer b;
+			int i;
 
 			/* pass the packet as parameter */
-			lua_pushlstring(L, packet->str + NET_HEADER_SIZE, packet->len - NET_HEADER_SIZE);
+			luaL_buffinit(L, &b);
+			/* iterate over the packets and append them all together */
+			for (i = 0; NULL != (packet = g_queue_peek_nth(recv_sock->recv_queue->chunks, i)); i++) {
+				luaL_addlstring(&b, packet->str + NET_HEADER_SIZE, packet->len - NET_HEADER_SIZE);
+			}
+			luaL_pushresult(&b);
 
 			if (lua_pcall(L, 1, 1, 0) != 0) {
 				/* hmm, the query failed */
@@ -1034,8 +1049,6 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
 				 *
 				 */
 	
-				con->client->packet_id++;
-
 				if (network_mysqld_con_lua_handle_proxy_response(con, con->config->lua_script)) {
 					/**
 					 * handling proxy.response failed
@@ -1091,32 +1104,20 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
  */
 NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 	GString *packet;
-	GList *chunk;
 	network_socket *recv_sock, *send_sock;
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
 	int proxy_query = 1;
 	network_mysqld_lua_stmt_ret ret;
+	
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query::enter");
 
 	send_sock = NULL;
 	recv_sock = con->client;
 	st->injected.sent_resultset = 0;
 
-	chunk = recv_sock->recv_queue->chunks->head;
-
-	if (recv_sock->recv_queue->chunks->length != 1) {
-		g_message("%s.%d: client-recv-queue-len = %d", __FILE__, __LINE__, recv_sock->recv_queue->chunks->length);
-	}
-	
-	packet = chunk->data;
-
-	if (packet->len != recv_sock->packet_len + NET_HEADER_SIZE) {
-		g_warning("%s: malformed packet, simply sending it on. raw length != protocol length (%d != %d)", G_STRLOC, (int)packet->len, (int)recv_sock->packet_len + NET_HEADER_SIZE);
-		return NETWORK_SOCKET_SUCCESS;
-	};
-
-	con->parse.len = recv_sock->packet_len;
-
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query::enter_lua");
 	ret = proxy_lua_read_query(con);
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query::leave_lua");
 
 	/**
 	 * if we disconnected in read_query_result() we have no connection open
@@ -1135,17 +1136,17 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 	switch (ret) {
 	case PROXY_NO_DECISION:
 	case PROXY_SEND_QUERY:
-		/* no injection, pass on the chunk as is */
-		send_sock->packet_id = recv_sock->packet_id;
-
-		network_queue_append(send_sock->send_queue, packet);
+		/* no injection, pass on the chunks as is */
+		while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) {
+			network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, packet);
+		}
 		con->resultset_is_needed = FALSE; /* we don't want to buffer the result-set */
 
 		break;
 	case PROXY_SEND_RESULT: 
 		proxy_query = 0;
 		
-		g_string_free(chunk->data, TRUE);
+		while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(packet, TRUE);
 
 		break; 
 	case PROXY_SEND_INJECTION: {
@@ -1154,25 +1155,23 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 		inj = g_queue_peek_head(st->injected.queries);
 		con->resultset_is_needed = inj->resultset_is_needed; /* let the lua-layer decide if we want to buffer the result or not */
 
-		/* there might be no query, if it was banned */
-		network_mysqld_queue_append(send_sock->send_queue, inj->query->str, inj->query->len, 0);
+		network_mysqld_queue_reset(send_sock);
+		network_mysqld_queue_append(send_sock, send_sock->send_queue, S(inj->query));
 
-		g_string_free(chunk->data, TRUE);
+		while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(packet, TRUE);
 
 		break; }
 	default:
 		g_error("%s.%d: ", __FILE__, __LINE__);
 	}
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
-	g_queue_delete_link(recv_sock->recv_queue->chunks, chunk);
-
 	if (proxy_query) {
 		con->state = CON_STATE_SEND_QUERY;
 	} else {
 		con->state = CON_STATE_SEND_QUERY_RESULT;
 		con->resultset_is_finished = TRUE; /* we don't have more too send */
 	}
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query::done");
 
 	return NETWORK_SOCKET_SUCCESS;
 }
@@ -1223,8 +1222,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_send_query_result) {
 		return NETWORK_SOCKET_SUCCESS;
 	}
 
-	con->parse.len = recv_sock->packet_len;
-
 	/* looks like we still have queries in the queue, 
 	 * push the next one 
 	 */
@@ -1243,7 +1240,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_send_query_result) {
 	g_assert(inj);
 	g_assert(send_sock);
 
-	network_mysqld_queue_append(send_sock->send_queue, S(inj->query), 0);
+	network_mysqld_queue_reset(send_sock);
+	network_mysqld_queue_append(send_sock, send_sock->send_queue, S(inj->query));
 
 	network_mysqld_con_reset_command_response_state(con);
 
@@ -1265,46 +1263,40 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_send_query_result) {
 NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query_result) {
 	int is_finished = 0;
 	network_packet packet;
-	GList *chunk;
 	network_socket *recv_sock, *send_sock;
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
 	injection *inj = NULL;
 
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::enter");
+
 	recv_sock = con->server;
 	send_sock = con->client;
 
-	chunk = recv_sock->recv_queue->chunks->tail;
-	packet.data = chunk->data;
+	/* check if the last packet is valid */
+	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
 	packet.offset = 0;
 
-	/**
-	 * check if we want to forward the statement to the client 
-	 *
-	 * if not, clean the send-queue 
-	 */
-
 	if (0 != st->injected.queries->length) {
 		inj = g_queue_peek_head(st->injected.queries);
 	}
 
-	if (inj && inj->ts_read_query_result_first.tv_sec == 0) {
+	if (inj && inj->ts_read_query_result_first == 0) {
 		/**
 		 * log the time of the first received packet
 		 */
-		g_get_current_time(&(inj->ts_read_query_result_first));
+		inj->ts_read_query_result_first = chassis_get_rel_microseconds();
+		/* g_get_current_time(&(inj->ts_read_query_result_first)); */
 	}
 
-	if (packet.data->len != recv_sock->packet_len + NET_HEADER_SIZE) return NETWORK_SOCKET_SUCCESS;
-
 	is_finished = network_mysqld_proto_get_query_result(&packet, con);
 	if (is_finished == -1) return NETWORK_SOCKET_ERROR; /* something happend, let's get out of here */
 
 	con->resultset_is_finished = is_finished;
 
-	network_queue_append(send_sock->send_queue, packet.data);
-
-	g_queue_delete_link(recv_sock->recv_queue->chunks, chunk);
-	recv_sock->packet_len = PACKET_LEN_UNSET;
+	/* copy the packet over to the send-queue if we don't need it */
+	if (!con->resultset_is_needed) {
+		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, g_queue_pop_tail(recv_sock->recv_queue->chunks));
+	}
 
 	if (is_finished) {
 		/**
@@ -1330,12 +1322,18 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query_result) {
 				inj->qstat.warning_count = com_query->warning_count;
 				inj->qstat.query_status  = com_query->query_status;
 			}
-			g_get_current_time(&(inj->ts_read_query_result_last));
+			inj->ts_read_query_result_last = chassis_get_rel_microseconds();
+			/* g_get_current_time(&(inj->ts_read_query_result_last)); */
 		}
+		
+		network_mysqld_queue_reset(recv_sock);
 
+		NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::enter_lua");
 		proxy_lua_read_query_result(con);
+		NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::leave_lua");
 
 		/** recv_sock might be != con->server now */
+		network_mysqld_queue_reset(send_sock);
 
 		/**
 		 * if the send-queue is empty, we have nothing to send
@@ -1348,6 +1346,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query_result) {
 			con->state = CON_STATE_READ_QUERY;
 		}
 	}
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::leave");
 	
 	return NETWORK_SOCKET_SUCCESS;
 }
@@ -1366,9 +1365,11 @@ static network_mysqld_lua_stmt_ret proxy_lua_connect_server(network_mysqld_con *
 	case REGISTER_CALLBACK_SUCCESS:
 		break;
 	case REGISTER_CALLBACK_LOAD_FAILED:
+		/* send packet-id 0 */
 		network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to load. Check the error log."));
 		return PROXY_SEND_RESULT;
 	case REGISTER_CALLBACK_EXECUTE_FAILED:
+		/* send packet-id 0 */
 		network_mysqld_con_send_error(con->client, C("MySQL Proxy Lua script failed to execute. Check the error log."));
 		return PROXY_SEND_RESULT;
 	}
@@ -1412,6 +1413,7 @@ static network_mysqld_lua_stmt_ret proxy_lua_connect_server(network_mysqld_con *
 				 * send a ERR packet
 				 */
 		
+				/* send packet-id 0 */
 				network_mysqld_con_send_error(con->client, C("(lua) handling proxy.response failed, check error-log"));
 			}
 
@@ -1612,6 +1614,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 		con->state = CON_STATE_READ_HANDSHAKE;
 	} else {
 		GString *auth_packet;
+
 		/**
 		 * send the old hand-shake packet
 		 */
@@ -1619,10 +1622,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 		auth_packet = g_string_new(NULL);
 		network_mysqld_proto_append_auth_challenge(auth_packet, con->server->challenge);
 
-		/* remove the idle-handler from the socket */
-		network_mysqld_queue_append(con->client->send_queue, 
-				S(auth_packet),
-			       	0); /* packet-id */
+		network_mysqld_queue_append(
+				con->client,
+				con->client->send_queue, 
+				S(auth_packet));
 
 		g_string_free(auth_packet, TRUE);
 		
@@ -1861,15 +1864,15 @@ static GOptionEntry * network_mysqld_proxy_plugin_get_options(chassis_plugin_con
 	/* make sure it isn't collected */
 	static GOptionEntry config_entries[] = 
 	{
-		{ "proxy-address",            0, 0, G_OPTION_ARG_STRING, NULL, "listening address:port of the proxy-server (default: :4040)", "<host:port>" },
+		{ "proxy-address",            'P', 0, G_OPTION_ARG_STRING, NULL, "listening address:port of the proxy-server (default: :4040)", "<host:port>" },
 		{ "proxy-read-only-backend-addresses", 
-					      0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, "address:port of the remote slave-server (default: not set)", "<host:port>" },
-		{ "proxy-backend-addresses",  0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, "address:port of the remote backend-servers (default: 127.0.0.1:3306)", "<host:port>" },
+					      'r', 0, G_OPTION_ARG_STRING_ARRAY, NULL, "address:port of the remote slave-server (default: not set)", "<host:port>" },
+		{ "proxy-backend-addresses",  'b', 0, G_OPTION_ARG_STRING_ARRAY, NULL, "address:port of the remote backend-servers (default: 127.0.0.1:3306)", "<host:port>" },
 		
 		{ "proxy-skip-profiling",     0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, NULL, "disables profiling of queries (default: enabled)", NULL },
 
 		{ "proxy-fix-bug-25371",      0, 0, G_OPTION_ARG_NONE, NULL, "fix bug #25371 (mysqld > 5.1.12) for older libmysql versions", NULL },
-		{ "proxy-lua-script",         0, 0, G_OPTION_ARG_FILENAME, NULL, "filename of the lua script (default: not set)", "<file>" },
+		{ "proxy-lua-script",         's', 0, G_OPTION_ARG_FILENAME, NULL, "filename of the lua script (default: not set)", "<file>" },
 		
 		{ "no-proxy",                 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, NULL, "don't start the proxy-module (default: enabled)", NULL },
 		
@@ -1936,6 +1939,7 @@ int network_mysqld_proxy_plugin_apply_config(chassis *chas, chassis_plugin_confi
 	if (0 != network_socket_bind(listen_sock)) {
 		return -1;
 	}
+	g_message("proxy listening on port %s", config->address);
 
 	for (i = 0; config->backend_addresses && config->backend_addresses[i]; i++) {
 		network_backends_add(g->backends, config->backend_addresses[i], BACKEND_TYPE_RW);
