@@ -130,45 +130,58 @@ static int network_mysqld_resultset_master_status(chassis *UNUSED_PARAM(chas), n
 	/* a data row */
 	while (NULL != (chunk = chunk->next)) {
 		network_packet packet;
-		guint8 status;
+		network_mysqld_lenenc_type lenenc_type;
 
 		packet.data = chunk->data;
 		packet.offset = 0;
 
-		err = err || network_mysqld_proto_get_int8(&packet, &status);
+		err = err || network_mysqld_proto_peek_lenenc_type(&packet, &lenenc_type);
+		if (err) break; /* proto error */
 
-		/* if we find the 2nd EOF packet we are done */
-		if (status == MYSQLD_PACKET_EOF &&
-		    packet.data->len < 10) break;
+		if (lenenc_type == NETWORK_MYSQLD_LENENC_TYPE_EOF) break; /* last packet */
 
 		for (i = 0; i < fields->len; i++) {
 			guint64 field_len;
-			err = err || network_mysqld_proto_get_lenenc_int(&packet, &field_len);
+			err = err || network_mysqld_proto_peek_lenenc_type(&packet, &lenenc_type);
 
-			if (i == 0) {
-				g_assert(field_len > 0);
-				/* Position */
-				
-				if (st->binlog_file) g_free(st->binlog_file);
+			switch (lenenc_type) {
+			case NETWORK_MYSQLD_LENENC_TYPE_INT:
+				err = err || network_mysqld_proto_get_lenenc_int(&packet, &field_len);
 
-				err = err || network_mysqld_proto_get_string_len(&packet, &st->binlog_file, field_len);
-			} else if (i == 1) {
-				/* is a string */
-				gchar *num;
+				if (i == 0) {
+					gchar *s;
+					/* Position */
 
-				g_assert(field_len > 0);
+					err = err || !(field_len > 0);
+					err = err || network_mysqld_proto_get_string_len(&packet, &s, field_len);
 
-				st->binlog_pos = 0;
+					if (!err) {
+						if (st->binlog_file) g_free(st->binlog_file);
+						st->binlog_file = s;
+					}
+				} else if (i == 1) {
+					/* is a string */
+					gchar *num;
 
-				err = err || network_mysqld_proto_get_string_len(&packet, &num, field_len);
-				st->binlog_pos = g_ascii_strtoull(num, NULL, 10);
-			} else {
-				err = err || network_mysqld_proto_skip(&packet, field_len);
+					err = err || !(field_len > 0);
+					err = err || network_mysqld_proto_get_string_len(&packet, &num, field_len);
+					if (!err) {
+						st->binlog_pos = g_ascii_strtoull(num, NULL, 10);
+					}
+				} else {
+					/* extra fields we don't expect */
+					err = err || network_mysqld_proto_skip(&packet, field_len);
+				}
+			default:
+				/* we don't expect a ERR, EOF or NULL here */
+				break;
 			}
 		}
 
-		g_message("reading binlog from: binlog-file: %s, binlog-pos: %d", 
-				st->binlog_file, st->binlog_pos);
+		if (!err) {
+			g_message("reading binlog from: binlog-file: %s, binlog-pos: %d", 
+					st->binlog_file, st->binlog_pos);
+		}
 	}
 
 	return err ? -1 : 0;
@@ -192,17 +205,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_handshake) {
 	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
 	packet.offset = 0;
 
-	if (packet.data->len != recv_sock->packet_len + NET_HEADER_SIZE) {
-		/**
-		 * packet is too short, looks nasty.
-		 *
-		 * report an error and let the core send a error to the 
-		 * client
-		 */
-
-		return NETWORK_SOCKET_ERROR;
-	}
-
 	err = err || network_mysqld_proto_skip_network_header(&packet);
 	if (err) return NETWORK_SOCKET_ERROR;
 
@@ -211,8 +213,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_handshake) {
 
 	g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
-	
 	if (err) {
 		network_mysqld_auth_challenge_free(shake);
 		return NETWORK_SOCKET_ERROR;
@@ -231,12 +231,17 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_handshake) {
 	}
 
 	if (config->mysqld_password) {
-		network_mysqld_proto_scramble(auth->response, shake->challenge, config->mysqld_password);
+		GString *hashed_password;
+
+		hashed_password = g_string_new(NULL);
+		network_mysqld_proto_password_hash(hashed_password, config->mysqld_password, strlen(config->mysqld_password));
+
+		network_mysqld_proto_password_scramble(auth->response, S(shake->challenge), S(hashed_password));
 	}
 
 	network_mysqld_proto_append_auth_response(auth_packet, auth);
 
-	network_mysqld_queue_append(send_sock->send_queue, S(auth_packet), send_sock->packet_id + 1);
+	network_mysqld_queue_append(send_sock, send_sock->send_queue, S(auth_packet));
 
 	network_mysqld_auth_response_free(auth);
 	network_mysqld_auth_challenge_free(shake);
@@ -260,14 +265,9 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_auth_result) {
 
 	recv_sock = con->server;
 
-	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
+	packet.data = g_queue_peek_head(recv_sock->recv_queue->chunks);
 	packet.offset = 0;
 
-	/* we aren't finished yet */
-	if (packet.data->len != recv_sock->packet_len + NET_HEADER_SIZE) {
-		return NETWORK_SOCKET_SUCCESS;
-	}
-
 	err = err || network_mysqld_proto_skip_network_header(&packet);
 	err = err || network_mysqld_proto_peek_int8(&packet, &status);
 	if (err) return NETWORK_SOCKET_ERROR;
@@ -301,12 +301,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_auth_result) {
 		return NETWORK_SOCKET_ERROR;
 	} 
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
-
 	g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
 	send_sock = con->server;
-	network_mysqld_queue_append(send_sock->send_queue, C(query_packet), 0);
+	network_mysqld_queue_append(send_sock, send_sock->send_queue, C(query_packet));
 
 	con->state = CON_STATE_SEND_QUERY;
 
@@ -352,7 +350,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_query_result) {
 	packet.data = chunk->data;
 	packet.offset = 0;
 
-	if (packet.data->len != recv_sock->packet_len + NET_HEADER_SIZE) return NETWORK_SOCKET_SUCCESS; /* packet isn't finished yet */
 #if 0
 	g_message("%s.%d: packet-len: %08x, packet-id: %d, command: COM_(%02x)", 
 			__FILE__, __LINE__,
@@ -400,10 +397,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_query_result) {
 		break;
 	}
 
-	network_mysqld_queue_append(send_sock->send_queue, 
+	network_mysqld_queue_append(send_sock,
+			send_sock->send_queue, 
 			packet.data->str + NET_HEADER_SIZE, 
-			packet.data->len - NET_HEADER_SIZE, 
-			recv_sock->packet_id);
+			packet.data->len - NET_HEADER_SIZE);
 
 	/* ... */
 	if (is_finished) {
@@ -437,7 +434,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_query_result) {
 			network_mysqld_proto_append_binlog_dump(query_packet, dump);
 		       	
 			send_sock = con->server;
-			network_mysqld_queue_append(send_sock->send_queue, query_packet->str, query_packet->len, 0);
+			network_mysqld_queue_append(send_sock, send_sock->send_queue, S(query_packet));
 
 			network_mysqld_binlog_dump_free(dump);
 		
@@ -460,8 +457,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_query_result) {
 	if (chunk->data) g_string_free(chunk->data, TRUE);
 	g_queue_delete_link(recv_sock->recv_queue->chunks, chunk);
 
-	recv_sock->packet_len = PACKET_LEN_UNSET;
-
 	return NETWORK_SOCKET_SUCCESS;
 }
 
