@@ -23,11 +23,17 @@
 #include <glib.h>
 
 #include "chassis-exports.h"
+
+/** @addtogroup chassis */
+/*@{*/
 /**
  * parse the configfile options into option entries
  *
  */
 CHASSIS_API int chassis_keyfile_to_options(GKeyFile *keyfile, const gchar *groupname, GOptionEntry *config_entries);
+CHASSIS_API int chassis_keyfile_resolve_path(const char *base_dir, GOptionEntry *config_entries);
+
+/*@}*/
 
 #endif
 
