@@ -16,66 +16,6 @@
 
  $%ENDLICENSE%$ */
  
-
-/**
- * \mainpage
- *
- * \section Architecture
- *
- * MySQL Proxy is based around the C10k problem as described by http://kegel.com/c10k.html
- *
- * This leads to some basic features
- * - 10.000 concurrent connections in one program
- * - spreading the load over several backends
- * - each backend might be able to only handle 100 connections (max_connections)
- * 
- * We can implement 
- * - reusing idling backend connections 
- * - splitting client connections into several backend connections
- *
- * Most of the magic is happening in the scripting layer provided by lua (http://lua.org/) which
- * was picked as it:
- *
- * - is very easy to embed
- * - is small (200kb stripped) and efficient (see http://shootout.alioth.debian.org/gp4/benchmark.php?test=all&lang=all)
- * - is easy to read and write
- *
- * \section a walk through the code
- *
- * To understand the code you basicly only have to know about the three files documented below:
- *
- * - chassis.c
- *   - main()
- *     -# command-line handling
- *     -# plugin loading
- *     -# logging
- * - network-mysqld.c
- *   - network_mysqld_thread() (supposed be called as thread)
- *     -# registers event-halders (event_set(..., network_mysqld_con_accept, ...))
- *     -# calls event_base_dispatch() [libevent] in the mainloop 
- *   - network_mysqld_con_accept()
- *     -# is called when the listen()ing socket gets a incoming connection
- *     -# sets the event-handler for the established connection (e.g. network_mysqld_proxy_connection_init())
- *     -# calls network_mysqld_con_handle() on the connection 
- *   - network_mysqld_con_handle() is the state-machine
- *     -# implements the states of the \ref protocol "MySQL Protocol"
- *     -# calls plugin functions (registered by e.g. network_mysqld_proxy_connection_init()) 
- * - network-mysqld-proxy.c
- *   - implements the \ref proxy_states "proxy specific states"
- *
- * The other files only help those based main modules to do their job:
- *
- * - network-mysqld-proto.c
- *   - the byte functions around the \ref proto "MySQL protocol"
- * - network-socket.c
- *   - basic socket struct 
- * - network-mysqld-table.c
- *   - internal tables to select from on the admin interface (to be removed) 
- * - network-conn-pool.c
- *   - a connection pool for server connections 
- */
-
-
 /** @file
  * the user-interface for the MySQL Proxy @see main()
  *
@@ -88,9 +28,6 @@
  * @todo move the SQL based help out into a lua script
  */
 
-
-#define SVN_REVISION "$Rev$"
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -112,8 +49,8 @@
 #include <io.h>      /* open() */
 #else
 #include <unistd.h>
-#include <sys/resource.h>
 #include <sys/wait.h>
+#include <sys/resource.h> /* for rusage in wait() */
 #endif
 
 #include <glib.h>
@@ -141,6 +78,8 @@
 #include "chassis-keyfile.h"
 #include "chassis-mainloop.h"
 #include "chassis-path.h"
+#include "chassis-limits.h"
+#include "chassis-filemode.h"
 
 #ifdef _WIN32
 static char **shell_argv;
@@ -300,9 +239,8 @@ static int proc_keepalive(int *child_exit_status) {
 					g_assert_not_reached();
 				}
 			} else if (-1 == exit_pid) {
-				if (EINTR == errno) {
-					/* EINTR is ok, all others bad */
-				} else {
+				/* EINTR is ok, all others bad */
+				if (EINTR != errno) {
 					/* how can this happen ? */
 					g_critical("%s: wait4(%d, ...) failed: %s (%d)",
 						G_STRLOC,
@@ -318,7 +256,7 @@ static int proc_keepalive(int *child_exit_status) {
 		}
 	}
 
-	return 1;
+	/* return 1; */ /* never reached, compiler complains */
 }
 #endif
 
@@ -365,6 +303,35 @@ static void sigsegv_handler(int G_GNUC_UNUSED signum) {
 	abort(); /* trigger a SIGABRT instead of just exiting */
 }
 
+static int chassis_setenv_lua(const char *key, const char *value) {
+	int r;
+#if _WIN32
+	/** on Win32 glib uses _wputenv to set the env variable,
+	 *  but Lua uses getenv. Those two don't see each other,
+	 *  so we use _putenv. Since we only set ASCII chars, this
+	 *  is safe.
+	 */
+	r = _putenv_s(key, value);
+#else
+	r = g_setenv(key, value, 1) ? 0 : -1; /* g_setenv() returns TRUE/FALSE */
+#endif
+
+	if (0 == r) {
+		/* the setenv() succeeded, double-check it */
+		if (!getenv(key)) {
+			/* check that getenv() returns what we did set */
+			g_critical("%s: setting %s = %s failed: (getenv() == NULL)", G_STRLOC,
+					key, value);
+		} else if (0 != strcmp(getenv(key), value)) {
+			g_critical("%s: setting %s = %s failed: (getenv() == %s)", G_STRLOC,
+					key, value,
+					getenv(key));
+		}
+	}
+
+	return r;
+}
+
 /**
  * This is the "real" main which is called both on Windows and UNIX platforms.
  * For the Windows service case, this will also handle the notifications and set
@@ -376,10 +343,11 @@ int main_cmdline(int argc, char **argv) {
 	static struct sigaction sigsegv_sa;
 #endif
 	/* read the command-line options */
-	GOptionContext *option_ctx;
+	GOptionContext *option_ctx = NULL;
 	GError *gerr = NULL;
 	guint i;
 	int exit_code = EXIT_SUCCESS;
+	const gchar *exit_location = G_STRLOC;
 	int print_version = 0;
 	int daemon_mode = 0;
 	gchar *user = NULL;
@@ -395,17 +363,17 @@ int main_cmdline(int argc, char **argv) {
 	guint invoke_dbg_on_crash = 0;
 	guint auto_restart = 0;
 	guint max_files_number = 8192;
-#ifndef _WIN32
-	struct rlimit max_files_rlimit;
+	gint event_thread_count = 1; /* there is always the main-thread */
+#ifdef WIN32
+	WSADATA wsaData;
 #endif
 	gchar *log_level = NULL;
 
 	GKeyFile *keyfile = NULL;
 	chassis_log *log;
+	char *lua_path = NULL;
+	char *lua_cpath = NULL;
 
-	/* holds argv[0] cleansed of the potential suffix on this platform (.exe on win32) */
-	gchar *executable_name = NULL;
-	
 	/* can't appear in the configfile */
 	GOptionEntry base_main_entries[] = 
 	{
@@ -431,6 +399,9 @@ int main_cmdline(int argc, char **argv) {
 		{ "log-backtrace-on-crash",   0, 0, G_OPTION_ARG_NONE, NULL, "try to invoke debugger on crash", NULL },
 		{ "keepalive",                0, 0, G_OPTION_ARG_NONE, NULL, "try to restart the proxy if it crashed", NULL },
 		{ "max-open-files",           0, 0, G_OPTION_ARG_INT, NULL, "maximum number of open files (ulimit -n)", NULL},
+		{ "event-threads",            0, 0, G_OPTION_ARG_INT, NULL, "number of event-handling threads (default: 1)", NULL},
+		{ "lua-path",                 0, 0, G_OPTION_ARG_STRING, NULL, "set the LUA_PATH", "<...>" },
+		{ "lua-cpath",                0, 0, G_OPTION_ARG_STRING, NULL, "set the LUA_CPATH", "<...>" },
 		
 		{ NULL,                       0, 0, G_OPTION_ARG_NONE,   NULL, NULL, NULL }
 	};
@@ -457,28 +428,40 @@ int main_cmdline(int argc, char **argv) {
 		g_error("loading modules is not supported on this platform");
 	}
 
-#ifdef HAVE_GTHREAD	
 	g_thread_init(NULL);
-#endif
 
 	log = chassis_log_new();
 	log->min_lvl = G_LOG_LEVEL_MESSAGE; /* display messages while parsing or loading plugins */
+	g_log_set_default_handler(chassis_log_func, log);
 
 #ifdef _WIN32
+	if (0 != WSAStartup(MAKEWORD( 2, 2 ), &wsaData)) {
+		g_critical("WSAStartup failed to initialize the socket library.\n");
+
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
+		goto exit_nicely;
+	}
 	if (win32_running_as_service) {
 		log->use_windows_applog = TRUE;
 		log->event_source_handle = RegisterEventSource(NULL, "mysql-monitor-agent");	/* TODO: get the actual executable name here */
 		if (!log->event_source_handle) {
 			int err = GetLastError();
 			g_critical("unhandled error-code (%d) for RegisterEventSource(), shutting down", err);
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
 	}
 #endif
-	g_log_set_default_handler(chassis_log_func, log);
 
-	srv = chassis_new();
+	/* may fail on library mismatch */
+	if (NULL == (srv = chassis_new())) {
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
+		goto exit_nicely;
+	}
+
 	srv->log = log; /* we need the log structure for the log-rotation */
 
 	i = 0;
@@ -501,6 +484,9 @@ int main_cmdline(int argc, char **argv) {
 	main_entries[i++].arg_data  = &(invoke_dbg_on_crash);
 	main_entries[i++].arg_data  = &(auto_restart);
 	main_entries[i++].arg_data  = &(max_files_number);
+	main_entries[i++].arg_data  = &(event_thread_count);
+	main_entries[i++].arg_data  = &(lua_path);
+	main_entries[i++].arg_data  = &(lua_cpath);
 
 	option_ctx = g_option_context_new("- MySQL App Shell");
 	g_option_context_add_main_entries(option_ctx, base_main_entries, GETTEXT_PACKAGE);
@@ -515,11 +501,17 @@ int main_cmdline(int argc, char **argv) {
 	if (FALSE == g_option_context_parse(option_ctx, &argc, &argv, &gerr)) {
 		g_critical("%s", gerr->message);
 		
-		exit_code = EXIT_FAILURE;
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
 
 	if (default_file) {
+		if (chassis_filemode_check(default_file) != 0) {
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
+			goto exit_nicely;
+		}
 		keyfile = g_key_file_new();
 		g_key_file_set_list_separator(keyfile, ',');
 
@@ -528,7 +520,8 @@ int main_cmdline(int argc, char **argv) {
 					default_file,
 					gerr->message);
 
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
 	}
@@ -538,7 +531,11 @@ int main_cmdline(int argc, char **argv) {
 	 * and print their version numbers, too. then we exit cleanly.
 	 */
 	if (print_version) {
-		printf("%s" CHASSIS_NEWLINE, PACKAGE_STRING); 
+		/* allow to pass down a build-tag at build-time which gets hard-coded into the binary */
+#ifndef CHASSIS_BUILD_TAG
+#define CHASSIS_BUILD_TAG PACKAGE_STRING
+#endif
+		printf("%s" CHASSIS_NEWLINE, CHASSIS_BUILD_TAG); 
 		printf("  glib2: %d.%d.%d" CHASSIS_NEWLINE, GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
 #ifdef HAVE_EVENT_H
 		printf("  libevent: %s" CHASSIS_NEWLINE, event_get_version());
@@ -556,13 +553,15 @@ int main_cmdline(int argc, char **argv) {
 	if (FALSE == g_option_context_parse(option_ctx, &argc, &argv, &gerr)) {
 		g_critical("%s", gerr->message);
 
-		exit_code = EXIT_FAILURE;
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
 
 	if (keyfile) {
 		if (chassis_keyfile_to_options(keyfile, "mysql-proxy", main_entries)) {
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
 	}
@@ -573,6 +572,7 @@ int main_cmdline(int argc, char **argv) {
 	if (!base_dir) {
 		base_dir = chassis_get_basedir(argv[0]);
 		if (!base_dir) {
+			g_critical("%s: Failed to get base directory.", G_STRLOC);
 			goto exit_nicely;
 		}
 
@@ -585,7 +585,8 @@ int main_cmdline(int argc, char **argv) {
 		   What do we do on Windows when called as a service?
 		 */
 		g_critical("--basedir option must be an absolute path, but was %s", base_dir);
-		exit_code = EXIT_FAILURE;
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
 
@@ -598,28 +599,36 @@ int main_cmdline(int argc, char **argv) {
 	 *
 	 * we want to derive it from the basedir ...
 	 */
-	if (!g_getenv(LUA_PATH)) {
+	if (lua_path) {
+		if (0 != chassis_setenv_lua(LUA_PATH, lua_path)) {
+			g_critical("%s: setting %s = %s failed: %s", G_STRLOC,
+					LUA_PATH, lua_path,
+					g_strerror(errno));
+		}
+		if (print_version) printf("    LUA_PATH: %s" CHASSIS_NEWLINE, lua_path);
+	} else if (!g_getenv(LUA_PATH)) {
 		gchar *path = g_build_filename(base_dir, "lib", "mysql-proxy", "lua", "?.lua", NULL);
-#if _WIN32
-		/** on Win32 glib uses _wputenv to set the env variable,
-		 *  but Lua uses getenv. Those two don't see each other,
-		 *  so we use _putenv. Since we only set ASCII chars, this
-		 *  is safe.
-		 */
-		gchar *env_path = g_strdup_printf("%s=%s", LUA_PATH, path);
-		_putenv(env_path);
-		g_free(env_path);
-#else
-		g_setenv(LUA_PATH, path, 1);
-#endif
 		if (print_version) printf("    LUA_PATH: %s" CHASSIS_NEWLINE, path);
-
+		
+		if (chassis_setenv_lua(LUA_PATH, path)) {
+			g_critical("%s: setting %s = %s failed: %s", G_STRLOC,
+					LUA_PATH, path,
+					g_strerror(errno));
+		}
+		
 		g_free(path);
 	} else {
 		if (print_version) printf("    LUA_PATH: %s" CHASSIS_NEWLINE, g_getenv(LUA_PATH));
 	}
 
-	if (!g_getenv(LUA_CPATH)) {
+	if (lua_cpath) {
+		if (chassis_setenv_lua(LUA_CPATH, lua_cpath)) {
+			g_critical("%s: setting %s = %s failed: %s", G_STRLOC,
+					LUA_CPATH, lua_cpath,
+					g_strerror(errno));
+		}
+		if (print_version) printf("    LUA_CPATH: %s" CHASSIS_NEWLINE, lua_cpath);
+	} else if (!g_getenv(LUA_CPATH)) {
 		/* each OS has its own way of declaring a shared-lib extension
 		 *
 		 * win32 has .dll
@@ -628,15 +637,17 @@ int main_cmdline(int argc, char **argv) {
 		 */ 
 #  if _WIN32
 		gchar *path = g_build_filename(base_dir, "bin", "lua-?." G_MODULE_SUFFIX, NULL);
-		gchar *env_path = g_strdup_printf("%s=%s", LUA_CPATH, path);
-		_putenv(env_path);
-		g_free(env_path);
 #  else
 		gchar *path = g_build_filename(base_dir, "lib", "mysql-proxy", "lua", "?." G_MODULE_SUFFIX, NULL);
-		g_setenv(LUA_CPATH, path, 1);
 #  endif
 		if (print_version) printf("    LUA_CPATH: %s" CHASSIS_NEWLINE, path);
 
+		if (chassis_setenv_lua(LUA_CPATH, path)) {
+			g_critical("%s: setting %s = %s failed: %s", G_STRLOC,
+					LUA_CPATH, path,
+					g_strerror(errno));
+		}
+
 		g_free(path);
 	} else {
 		if (print_version) printf("    LUA_CPATH: %s" CHASSIS_NEWLINE, g_getenv(LUA_CPATH));
@@ -650,8 +661,6 @@ int main_cmdline(int argc, char **argv) {
 #ifdef HAVE_SIGACTION
 	/* register the sigsegv interceptor */
 
-	g_set_prgname(argv[0]); /* for g_on_error_stack_frame() */
-
 	memset(&sigsegv_sa, 0, sizeof(sigsegv_sa));
 	sigsegv_sa.sa_handler = sigsegv_handler;
 	sigemptyset(&sigsegv_sa.sa_mask);
@@ -681,9 +690,9 @@ int main_cmdline(int argc, char **argv) {
 	 * from the plugins, thus we need to fix them up before
 	 * dealing with all the rest.
 	 */
-	chassis_resolve_path(srv, &log->log_filename);
-	chassis_resolve_path(srv, &pid_file);
-	chassis_resolve_path(srv, &plugin_dir);
+	chassis_resolve_path(srv->base_dir, &log->log_filename);
+	chassis_resolve_path(srv->base_dir, &pid_file);
+	chassis_resolve_path(srv->base_dir, &plugin_dir);
 
 	if (log->log_filename) {
         gboolean turned_off_syslog = FALSE;
@@ -694,7 +703,8 @@ int main_cmdline(int argc, char **argv) {
 		if (0 == chassis_log_open(log)) {
 			g_critical("can't open log-file '%s': %s", log->log_filename, g_strerror(errno));
 
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
         if (turned_off_syslog)
@@ -706,7 +716,8 @@ int main_cmdline(int argc, char **argv) {
 		if (0 != chassis_log_set_level(log, log_level)) {
 			g_critical("--log-level=... failed, level '%s' is unknown ", log_level);
 
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
 	} else {
@@ -716,39 +727,11 @@ int main_cmdline(int argc, char **argv) {
 
 	/* if not plugins are specified, load admin and proxy */
 	if (!plugin_names) {
-		plugin_names = g_new0(char *, 4); /* make sure we allocate _enough_ memory */
+		plugin_names = g_new(char *, 3);
 
-#define IS_PNAME(pname) \
-		((strlen(executable_name) >= sizeof(pname) - 1) && \
-		0 == strcmp(executable_name + strlen(executable_name) - (sizeof(pname) - 1), pname) \
-		)
-#ifdef WIN32
-		/* on Windows allow for the executable and -svc suffix */
-		if (g_str_has_suffix(argv[0], "-svc.exe")) {
-			executable_name = g_strndup(argv[0], strlen(argv[0]) - (sizeof("-svc.exe") - 1));
-		} else if (g_str_has_suffix(argv[0], ".exe")) {
-			executable_name = g_strndup(argv[0], strlen(argv[0]) - (sizeof(".exe") - 1));
-		} else {
-			executable_name = g_strdup(argv[0]);
-		}
-#else
-		executable_name = argv[0];
-#endif
-		/* check what we are called as */
-		if (IS_PNAME("mysql-proxy")) {
-			plugin_names[0] = g_strdup("admin");
-			plugin_names[1] = g_strdup("proxy");
-			plugin_names[2] = NULL;
-		} else if (IS_PNAME("mysql-cli")) {
-			plugin_names[0] = g_strdup("cli");
-			plugin_names[1] = NULL;
-		}
-#ifdef WIN32
-		/* cleanup executable_name since we dup'ed it */
-		if (executable_name) {
-			g_free(executable_name);
-		}
-#endif
+		plugin_names[0] = g_strdup("admin");
+		plugin_names[1] = g_strdup("proxy");
+		plugin_names[2] = NULL;
 	}
 
 	/* load the plugins */
@@ -764,13 +747,20 @@ int main_cmdline(int argc, char **argv) {
 #ifndef SHARED_LIBRARY_SUFFIX
 #define SHARED_LIBRARY_SUFFIX G_MODULE_SUFFIX
 #endif
-		char *plugin_filename = g_strdup_printf("%s%c%s%s.%s", 
+		char *plugin_filename;
+		/* skip trying to load a plugin when the parameter was --plugins= 
+		   that will never work...
+		*/
+		if (!g_strcmp0("", plugin_names[i])) {
+			continue;
+		}
+		plugin_filename = g_strdup_printf("%s%c%s%s.%s", 
 				plugin_dir, 
 				G_DIR_SEPARATOR, 
 				G_MODULE_PREFIX,
 				plugin_names[i],
 				SHARED_LIBRARY_SUFFIX);
-		
+
 		p = chassis_plugin_load(plugin_filename);
 		g_free(plugin_filename);
 
@@ -785,7 +775,8 @@ int main_cmdline(int argc, char **argv) {
 		
 		if (NULL == p) {
 			g_critical("setting --plugin-dir=<dir> might help");
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
 
@@ -807,51 +798,28 @@ int main_cmdline(int argc, char **argv) {
 			if (FALSE == g_option_context_parse(option_ctx, &argc, &argv, &gerr)) {
 				g_critical("%s", gerr->message);
 		
-				exit_code = EXIT_FAILURE;
+				exit_code = EXIT_FAILURE; 
+				exit_location = G_STRLOC;
 				goto exit_nicely;
 			}
 	
 			if (keyfile) {
 				if (chassis_keyfile_to_options(keyfile, "mysql-proxy", config_entries)) {
-					exit_code = EXIT_FAILURE;
+					exit_code = EXIT_FAILURE; 
+					exit_location = G_STRLOC;
 					goto exit_nicely;
 				}
 			}
-			/* check for relative paths among the newly added options
-			 * and resolve them to an absolute path if we have --basedir
-			 */
-			if (srv->base_dir) {
-				int entry_idx;
-				for (entry_idx = 0; config_entries[entry_idx].long_name; entry_idx++) {
-					GOptionEntry entry = config_entries[entry_idx];
-					
-					switch(entry.arg) {
-					case G_OPTION_ARG_FILENAME: {
-						gchar **data = entry.arg_data;
-						chassis_resolve_path(srv, data);
-						break;
-					}
-					case G_OPTION_ARG_FILENAME_ARRAY: {
-						gchar ***data = entry.arg_data;
-						gchar **files = *data;
-						if (NULL != files) {
-							gint j;
-							for (j = 0; files[j]; j++) chassis_resolve_path(srv, &files[j]);
-						}
-						break;
-					}
-					default:
-						/* ignore other option types */
-						break;
-					}
-				}
-			}
+
+			/* resolve the path names for these config entries */
+			chassis_keyfile_resolve_path(srv->base_dir, config_entries); 
 		}
 	}
 
 	/* if we only print the version numbers, exit and don't do any more work */
 	if (print_version) {
 		exit_code = EXIT_SUCCESS;
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
 
@@ -875,7 +843,8 @@ int main_cmdline(int argc, char **argv) {
 					);
 		}
 		
-		exit_code = EXIT_FAILURE;
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
 
@@ -886,9 +855,21 @@ int main_cmdline(int argc, char **argv) {
 	if (argc > 1) {
 		g_critical("unknown option: %s", argv[1]);
 
-		exit_code = EXIT_FAILURE;
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
+
+	/* make sure that he max-thread-count isn't negative */
+	if (event_thread_count < 1) {
+		g_critical("--event-threads has to be >= 1, is %d", event_thread_count);
+
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
+		goto exit_nicely;
+	}
+
+	srv->event_thread_count = event_thread_count;
 	
 #ifndef _WIN32	
 	signal(SIGPIPE, SIG_IGN);
@@ -907,7 +888,8 @@ int main_cmdline(int argc, char **argv) {
 			exit_code = child_exit_status;
 			goto exit_nicely;
 		} else if (ret < 0) {
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		} else {
 			/* we are the child, go on */
@@ -928,7 +910,8 @@ int main_cmdline(int argc, char **argv) {
 					pid_file,
 					g_strerror(errno));
 
-			exit_code = EXIT_FAILURE;
+			exit_code = EXIT_FAILURE; 
+			exit_location = G_STRLOC;
 			goto exit_nicely;
 		}
 
@@ -955,45 +938,15 @@ int main_cmdline(int argc, char **argv) {
 	 */
 	srv->user = g_strdup(user);
 
-#ifdef _WIN32
-	g_debug("%s: current maximum number of open stdio file descriptors: %d", G_STRLOC, _getmaxstdio());
-	if (max_files_number > 2048) {
-		g_warning("%s: Windows only allows a maximum number of 2048 open files for stdio, using that value instead of %d", G_STRLOC, max_files_number);
-		max_files_number = 2048;
+	if (chassis_set_fdlimit(max_files_number)) {
+		/* do we want to exit or just go on */
 	}
-	if (-1 == _setmaxstdio(max_files_number)) {
-		g_critical("%s: failed to increase the maximum number of open files for stdio: %s (%d)", G_STRLOC, g_strerror(errno), errno);
-	} else {
-		g_debug("%s: set new limit of open files for stdio to %d", G_STRLOC, _getmaxstdio());
-	}
-#else
-	if (-1 == getrlimit(RLIMIT_NOFILE, &max_files_rlimit)) {
-		g_warning("%s: cannot get limit of open files for this process. %s (%d)",
-				  G_STRLOC, g_strerror(errno), errno);
-	} else {
-		rlim_t soft_limit = max_files_rlimit.rlim_cur;
-		g_debug("%s: current RLIMIT_NOFILE = %llu (hard: %llu)", G_STRLOC, max_files_rlimit.rlim_cur, max_files_rlimit.rlim_max);
-
-		max_files_rlimit.rlim_cur = max_files_number;
-
-		g_debug("%s: trying to set new RLIMIT_NOFILE = %llu (hard: %llu)", G_STRLOC, max_files_rlimit.rlim_cur, max_files_rlimit.rlim_max);
-		if (-1 == setrlimit(RLIMIT_NOFILE, &max_files_rlimit)) {
-			g_critical("%s: could not raise RLIMIT_NOFILE to %u, %s (%d). Current limit still %llu.", G_STRLOC, max_files_number, g_strerror(errno), errno, soft_limit);
-		} else {
-			if (-1 == getrlimit(RLIMIT_NOFILE, &max_files_rlimit)) {
-				g_warning("%s: cannot get limit of open files for this process. %s (%d)",
-						  G_STRLOC, g_strerror(errno), errno);
-			} else {
-				g_debug("%s: set new RLIMIT_NOFILE = %llu (hard: %llu)", G_STRLOC, max_files_rlimit.rlim_cur, max_files_rlimit.rlim_max);
-			}
-		}
-	}
-#endif
 
 	if (chassis_mainloop(srv)) {
 		/* looks like we failed */
-
-		exit_code = EXIT_FAILURE;
+		g_critical("%s: Failure from chassis_mainloop. Shutting down.", G_STRLOC);
+		exit_code = EXIT_FAILURE; 
+		exit_location = G_STRLOC;
 		goto exit_nicely;
 	}
 
@@ -1001,10 +954,13 @@ exit_nicely:
 	/* necessary to set the shutdown flag, because the monitor will continue
 	 * to schedule timers otherwise, causing an infinite loop in cleanup
 	 */
-	chassis_set_shutdown();
-	
+	if (!exit_code) {
+		exit_location = G_STRLOC;
+	}
+	chassis_set_shutdown_location(exit_location);
+
 	if (!print_version) {
-		g_message("shutting down normally"); /* add a tag to the logfile */
+		g_message("shutting down normally, exit code is: %d", exit_code); /* add a tag to the logfile */
 	}
 
 #ifdef _WIN32
@@ -1025,12 +981,12 @@ exit_nicely:
 	if (plugin_dir) g_free(plugin_dir);
 
 	if (plugin_names) {
-		for (i = 0; plugin_names[i]; i++) {
-			g_free(plugin_names[i]);
-		}
-		g_free(plugin_names);
+		g_strfreev(plugin_names);
 	}
 
+	if (lua_path) g_free(lua_path);
+	if (lua_cpath) g_free(lua_cpath);
+
 	chassis_log_free(log);
 	
 #ifdef _WIN32
@@ -1112,7 +1068,7 @@ int main_win32(int argc, char **argv) {
 		{ NULL, NULL } 
 	};
 
-	if (0 != WSAStartup(MAKEWORD( 1, 1 ), &wsaData)) {
+	if (0 != WSAStartup(MAKEWORD( 2, 2 ), &wsaData)) {
 		g_critical("WSAStartup failed to initialize the socket library.\n");
 
 		return -1;
