@@ -43,6 +43,10 @@ static int proxy_lua_panic (lua_State *L);
 
 static void *chassis_lua_alloc(void *userdata, void *ptr, size_t osize, size_t nsize);
 
+/**
+ * @deprecated will be removed in 1.0
+ * @see lua_scope_new()
+ */
 lua_scope *lua_scope_init(void) {
 	return lua_scope_new();
 }
@@ -58,9 +62,7 @@ lua_scope *lua_scope_new(void) {
 	lua_atpanic(sc->L, proxy_lua_panic);
 #endif
 
-#ifdef HAVE_GTHREAD
 	sc->mutex = g_mutex_new();
-#endif
 
 	return sc;
 }
@@ -84,19 +86,15 @@ void lua_scope_free(lua_scope *sc) {
 
 	lua_close(sc->L);
 #endif
-#ifdef HAVE_GTHREAD
 	g_mutex_free(sc->mutex);
-#endif
 
 	g_free(sc);
 }
 
 void lua_scope_get(lua_scope *sc, const char G_GNUC_UNUSED* pos) {
-#ifdef HAVE_GTHREAD
 /*	g_warning("%s: === waiting for lua-scope", pos); */
 	g_mutex_lock(sc->mutex);
 /*	g_warning("%s: +++ got lua-scope", pos); */
-#endif
 #ifdef HAVE_LUA_H
 	sc->L_top = lua_gettop(sc->L);
 #endif
@@ -111,10 +109,9 @@ void lua_scope_release(lua_scope *sc, const char* pos) {
 	}
 #endif
 
-#ifdef HAVE_GTHREAD
 	g_mutex_unlock(sc->mutex);
 /*	g_warning("%s: --- released lua scope", pos); */
-#endif
+
 	return;
 }
 
@@ -372,7 +369,7 @@ void proxy_lua_dumpstack(lua_State *L) {
 				printf("'%s'", lua_tostring(L, i));
 				break;
 			case LUA_TBOOLEAN:
-				printf(lua_toboolean(L, i) ? "true" : "false");
+				printf("%s", lua_toboolean(L, i) ? "true" : "false");
 				break;
 			case LUA_TNUMBER:
 				printf("'%g'", lua_tonumber(L, i));
@@ -461,53 +458,47 @@ static int proxy_lua_panic (lua_State *L) {
  * @param nsize the requested size of the block
  */
 static void* chassis_lua_alloc(void G_GNUC_UNUSED *userdata, void *ptr, size_t osize, size_t nsize) {
+	gpointer p;
+	gint cur_size;
+
 	/* the free case */
 	if (nsize == 0) {
 		if (osize != 0) {
 			CHASSIS_STATS_FREE_INC_NAME(lua_mem);
 			CHASSIS_STATS_ADD_NAME(lua_mem_bytes, -osize);
 			g_free(ptr);
-			return NULL;
-		} else /* osize == 0 */ {
-			return NULL;
 		}
-	} else /* nsize != 0 */ {
-		/* track the maximum of the mem-usage inside lua
-		 *
-		 * g_atomic_... works against signed integers, but we actually would need something bigger to be safe
-		 *
-		 * stats may be wrong if the lua-mem-* counters actually go about MAX_INT 
-		 */
-		if (osize == 0) { 		/* the plain malloc case */
-			gint cur_size;
-			CHASSIS_STATS_ALLOC_INC_NAME(lua_mem);
-			CHASSIS_STATS_ADD_NAME(lua_mem_bytes, nsize);
-			
-			cur_size = CHASSIS_STATS_GET_NAME(lua_mem_bytes);
-			if (cur_size > CHASSIS_STATS_GET_NAME(lua_mem_bytes_max)) {
-				CHASSIS_STATS_SET_NAME(lua_mem_bytes_max, cur_size);
-			}
-			return g_malloc(nsize);
-		} else /* osize != 0 */ {	/* the realloc case */
-			gpointer p = g_realloc(ptr, nsize);
-			gint cur_size;
-
-			if (!p) return p;
-			
-			CHASSIS_STATS_ADD_NAME(lua_mem_bytes, nsize - osize); /* might be negative if Lua tries to shrink something */
-
-			cur_size = CHASSIS_STATS_GET_NAME(lua_mem_bytes);
-			if (cur_size > CHASSIS_STATS_GET_NAME(lua_mem_bytes_max)) {
-				CHASSIS_STATS_SET_NAME(lua_mem_bytes_max, cur_size);
-			}
-			
-			return p;
+		return NULL;
+	} 
+	/* track the maximum of the mem-usage inside lua
+	 *
+	 * g_atomic_... works against signed integers, but we actually would need something bigger to be safe
+	 *
+	 * stats may be wrong if the lua-mem-* counters actually go about MAX_INT 
+	 */
+	if (osize == 0) { 		/* the plain malloc case */
+		CHASSIS_STATS_ALLOC_INC_NAME(lua_mem);
+		CHASSIS_STATS_ADD_NAME(lua_mem_bytes, nsize);
+		
+		cur_size = CHASSIS_STATS_GET_NAME(lua_mem_bytes);
+		if (cur_size > CHASSIS_STATS_GET_NAME(lua_mem_bytes_max)) {
+			CHASSIS_STATS_SET_NAME(lua_mem_bytes_max, cur_size);
 		}
+		return g_malloc(nsize);
+	} 
+
+	p = g_realloc(ptr, nsize);
+
+	if (!p) return p;
+	
+	CHASSIS_STATS_ADD_NAME(lua_mem_bytes, nsize - osize); /* might be negative if Lua tries to shrink something */
+
+	cur_size = CHASSIS_STATS_GET_NAME(lua_mem_bytes);
+	if (cur_size > CHASSIS_STATS_GET_NAME(lua_mem_bytes_max)) {
+		CHASSIS_STATS_SET_NAME(lua_mem_bytes_max, cur_size);
 	}
-	/* the ifs should cover all cases demanded of the lua allocator function, if not, crash hard */
-	g_critical("%s: userdata = %p, ptr = %p, osize = %"G_GSIZE_FORMAT", nsize = %"G_GSIZE_FORMAT, G_STRLOC, userdata, ptr, osize, nsize);
-	g_assert_not_reached();
-	return NULL;
+	
+	return p;
 }
 
 
