@@ -520,6 +520,8 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 					&pre_bits,
 					event->event.row_event.null_bits_len);
 
+			if (err) break;
+
 			pre_fields = network_mysqld_proto_fields_new_full(tbl->fields, 
 					pre_bits, 
 					event->event.row_event.null_bits_len);
@@ -1019,9 +1021,7 @@ int main(int argc, char **argv) {
 # endif
 #endif
 
-#ifdef HAVE_GTHREAD	
 	g_thread_init(NULL);
-#endif
 
 	log = chassis_log_new();
 	log->min_lvl = G_LOG_LEVEL_MESSAGE; /* display messages while parsing or loading plugins */
