@@ -43,6 +43,13 @@
 
 #include "network-exports.h"
 
+#ifdef __hpux
+/* see http://curl.haxx.se/mail/lib-2009-04/0287.html */
+typedef int network_socklen_t;
+#else
+typedef socklen_t network_socklen_t;
+#endif
+
 typedef struct {
 	union {
 		struct sockaddr_in ipv4;
@@ -55,14 +62,14 @@ typedef struct {
 
 	GString *name; 
 
-	socklen_t len;
+	network_socklen_t len;
 } network_address;
 
 NETWORK_API network_address *network_address_new(void);
 NETWORK_API void network_address_free(network_address *);
 NETWORK_API void network_address_reset(network_address *addr);
 NETWORK_API network_address *network_address_copy(network_address *dst, network_address *src);
-NETWORK_API gint network_address_set_address(network_address *addr, gchar *address);
+NETWORK_API gint network_address_set_address(network_address *addr, const gchar *address);
 NETWORK_API gint network_address_refresh_name(network_address *addr);
 NETWORK_API gint network_address_is_local(network_address *dst_addr, network_address *src_addr);
 
