@@ -22,6 +22,8 @@
 
 #if defined(mysql_chassis_proxy_EXPORTS)
 #define NETWORK_API __declspec(dllexport)
+#elif defined(mysql_chassis_proxy_STATIC)	/* In test cases, define this to prevent linker warnings on Win32 */
+#define NETWORK_API 
 #else
 #define NETWORK_API extern __declspec(dllimport)
 #endif
