@@ -340,7 +340,7 @@ network_mysqld_register_callback_ret network_mysqld_con_lua_register_callback(ne
 	q_p = lua_newuserdata(L, sizeof(GQueue *));               /* (sp += 1) 3 */
 	*q_p = st->injected.queries;
 
-	/**
+	/*
 	 * proxy.queries
 	 *
 	 * implement a queue
@@ -361,7 +361,7 @@ network_mysqld_register_callback_ret network_mysqld_con_lua_register_callback(ne
 
 	lua_setfield(L, -2, "queries"); /* proxy.queries = <userdata> */
 
-	/**
+	/*
 	 * proxy.connection is (mostly) read-only
 	 *
 	 * .thread_id  = ... thread-id against this server
@@ -377,7 +377,7 @@ network_mysqld_register_callback_ret network_mysqld_con_lua_register_callback(ne
 
 	lua_setfield(L, -2, "connection"); /* proxy.connection = <udata>     (sp -= 1) */
 
-	/**
+	/*
 	 * proxy.response knows 3 fields with strict types:
 	 *
 	 * .type = <int>
@@ -840,7 +840,8 @@ int network_mysqld_con_lua_handle_proxy_response(network_mysqld_con *con, const
 			if (lua_isstring(L, -1)) { /** proxy.response.packets[i] */
 				str = lua_tolstring(L, -1, &str_len);
 
-				network_mysqld_queue_append(con->client->send_queue, str, str_len, con->client->packet_id++);
+				network_mysqld_queue_append(con->client, con->client->send_queue,
+						str, str_len);
 	
 				lua_pop(L, 1); /* pop value */
 			} else if (lua_isnil(L, -1)) {
@@ -856,6 +857,8 @@ int network_mysqld_con_lua_handle_proxy_response(network_mysqld_con *con, const
 
 		lua_pop(L, 1); /* .packets */
 
+		network_mysqld_queue_reset(con->client); /* reset the packet-id checks */
+
 		break; }
 	default:
 		g_message("proxy.response.type is unknown: %d", resp_type);
