@@ -634,31 +634,37 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 GList *network_mysqld_proto_get_fielddefs(GList *chunk, GPtrArray *fields) {
 	network_packet packet;
 	guint64 field_count;
-	guint8 type;
 	guint i;
 	int err = 0;
 	guint32 capabilities = CLIENT_PROTOCOL_41;
+	network_mysqld_lenenc_type lenenc_type;
     
-	/*
-	 * read(6, "\1\0\0\1", 4)                  = 4
-	 * read(6, "\2", 1)                        = 1
-	 * read(6, "6\0\0\2", 4)                   = 4
-	 * read(6, "\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0", 54) = 54
-	 * read(6, "&\0\0\3", 4)                   = 4
-	 * read(6, "\3def\0\6STATUS\0\5Value\5Value\f\10\0\0\2\0\0\375\1\0\0\0\0", 38) = 38
-	 * read(6, "\5\0\0\4", 4)                  = 4
-	 * read(6, "\376\0\0\"\0", 5)              = 5
-	 * read(6, "\23\0\0\5", 4)                 = 4
-	 * read(6, "\17Aborted_clients\00298", 19) = 19
-	 *
-	 */
-
 	packet.data = chunk->data;
 	packet.offset = 0;
 
 	err = err || network_mysqld_proto_skip_network_header(&packet);
 	
+	err = err || network_mysqld_proto_peek_lenenc_type(&packet, &lenenc_type);
+
+	if (err) return NULL; /* packet too short */
+
+	/* make sure that we have a valid length-encoded integer here */
+	switch (lenenc_type) {
+	case NETWORK_MYSQLD_LENENC_TYPE_INT:
+		break;
+	default:
+		/* we shouldn't be here, we expected to get a valid length-encoded field count */
+		return NULL;
+	}
+	
 	err = err || network_mysqld_proto_get_lenenc_int(&packet, &field_count);
+	
+	if (err) return NULL; /* packet to short */
+
+	if (field_count == 0) {
+		/* shouldn't happen, the upper layer should have checked that this is a OK packet */
+		return NULL;
+	}
     
 	/* the next chunk, the field-def */
 	for (i = 0; i < field_count; i++) {
@@ -713,14 +719,14 @@ GList *network_mysqld_proto_get_fielddefs(GList *chunk, GPtrArray *fields) {
 				err = err || network_mysqld_proto_get_int8(&packet, &flags);
 				err = err || network_mysqld_proto_get_int8(&packet, (guint8 *)&field->decimals);
 
-				field->flags = flags;
+				if (!err) field->flags = flags;
 			} else {
 				/* well */
 			}
 
 		}
         
-		g_ptr_array_add(fields, field);
+		g_ptr_array_add(fields, field); /* even if we had an error, append it so that we can free it later */
 
 		if (err) return NULL;
 	}
@@ -735,8 +741,8 @@ GList *network_mysqld_proto_get_fielddefs(GList *chunk, GPtrArray *fields) {
 	
 	err = err || network_mysqld_proto_skip_network_header(&packet);
 
-	err = err || network_mysqld_proto_get_int8(&packet, &type); /* the byte after the net-header is a EOF */
-	err = err || (type != MYSQLD_PACKET_EOF);
+	err = err || network_mysqld_proto_peek_lenenc_type(&packet, &lenenc_type);
+	err = err || (lenenc_type != NETWORK_MYSQLD_LENENC_TYPE_EOF);
 
 	if (err) return NULL;
     
@@ -880,10 +886,7 @@ int network_mysqld_proto_get_err_packet(network_packet *packet, network_mysqld_e
  * @note the sqlstate has to match the SQL standard. If no matching SQL state is known, leave it at NULL
  *
  * @param packet      network packet
- * @param errmsg      the error message
- * @param errmsg_len  byte-len of the error-message
- * @param errorcode   mysql error-code we want to send
- * @param sqlstate    if none-NULL, 5-char SQL state to send, if NULL, default SQL state is used
+ * @param err_packet  the error structure
  *
  * @return 0 on success
  */
@@ -1230,9 +1233,14 @@ int network_mysqld_proto_get_auth_response(network_packet *packet, network_mysql
 		err = err || network_mysqld_proto_get_int16(packet, &l_cap);
 		err = err || network_mysqld_proto_get_int24(packet, &auth->max_packet_size);
 		err = err || network_mysqld_proto_get_gstring(packet, auth->username);
-		err = err || network_mysqld_proto_get_gstring(packet, auth->response);
+		/* there may be no password sent */
+		if (packet->data->len != packet->offset) {
+			err = err || network_mysqld_proto_get_gstring(packet, auth->response);
+		}
 
-		auth->capabilities = l_cap;
+		if (!err) {
+			auth->capabilities = l_cap;
+		}
 	}
 
 	return err ? -1 : 0;
@@ -1245,27 +1253,35 @@ int network_mysqld_proto_append_auth_response(GString *packet, network_mysqld_au
 	int i;
 
 	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
-		g_critical("%s: auth-capatilities = 0x%08x (require CLIENT_PROTOCOL_41)", G_STRLOC, auth->capabilities);
-		return -1;
-	}
+		network_mysqld_proto_append_int16(packet, auth->capabilities);
+		network_mysqld_proto_append_int24(packet, auth->max_packet_size); /* max-allowed-packet */
 
-	network_mysqld_proto_append_int32(packet, auth->capabilities);
-	network_mysqld_proto_append_int32(packet, auth->max_packet_size); /* max-allowed-packet */
-	
-	network_mysqld_proto_append_int8(packet, auth->charset); /* charset */
-
-	for (i = 0; i < 23; i++) { /* filler */
-		network_mysqld_proto_append_int8(packet, 0x00);
-	}
-
-	if (auth->username->len) g_string_append_len(packet, S(auth->username));
-	network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
-
-	/* scrambled password */
-	network_mysqld_proto_append_lenenc_string_len(packet, S(auth->response));
-	if (auth->database->len) {
-		g_string_append_len(packet, S(auth->database));
+		if (auth->username->len) g_string_append_len(packet, S(auth->username));
 		network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+
+		if (auth->response->len) {
+			g_string_append_len(packet, S(auth->response));
+			network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+		}
+	} else {
+		network_mysqld_proto_append_int32(packet, auth->capabilities);
+		network_mysqld_proto_append_int32(packet, auth->max_packet_size); /* max-allowed-packet */
+		
+		network_mysqld_proto_append_int8(packet, auth->charset); /* charset */
+
+		for (i = 0; i < 23; i++) { /* filler */
+			network_mysqld_proto_append_int8(packet, 0x00);
+		}
+
+		if (auth->username->len) g_string_append_len(packet, S(auth->username));
+		network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+
+		/* scrambled password */
+		network_mysqld_proto_append_lenenc_string_len(packet, S(auth->response));
+		if (auth->database->len) {
+			g_string_append_len(packet, S(auth->database));
+			network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+		}
 	}
 
 	return 0;
