@@ -47,7 +47,7 @@
 #define CRASHME() do { char *_crashme = NULL; *_crashme = 0; } while(0);
 
 /**
- * a handy marco for constant strings 
+ * a handy macro for constant strings 
  */
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
@@ -60,14 +60,51 @@
  * */
 /*@{*/
 
+/**
+ * extract the type of the object that is placed where a length-encoded string is expected
+ *
+ * reads a byte from the packet and checks if it either a:
+ * - integer
+ * - NULL
+ * - a ERR packet
+ * - a EOF packet
+ */
+int network_mysqld_proto_peek_lenenc_type(network_packet *packet, network_mysqld_lenenc_type *type) {
+	guint off = packet->offset;
+	unsigned char *bytestream = (unsigned char *)packet->data->str;
+
+	g_return_val_if_fail(off < packet->data->len, -1);
+
+	if (bytestream[off] < 251) { /* */
+		*type = NETWORK_MYSQLD_LENENC_TYPE_INT;
+	} else if (bytestream[off] == 251) { /* NULL */
+		*type = NETWORK_MYSQLD_LENENC_TYPE_NULL;
+	} else if (bytestream[off] == 252) { /* 2 byte length*/
+		*type = NETWORK_MYSQLD_LENENC_TYPE_INT;
+	} else if (bytestream[off] == 253) { /* 3 byte */
+		*type = NETWORK_MYSQLD_LENENC_TYPE_INT;
+	} else if (bytestream[off] == 254) { /* 8 byte OR EOF */
+		if (off == 4 && 
+		    packet->data->len - packet->offset < 8) {
+			*type = NETWORK_MYSQLD_LENENC_TYPE_EOF;
+		} else {
+			*type = NETWORK_MYSQLD_LENENC_TYPE_INT;
+		}
+	} else {
+		*type = NETWORK_MYSQLD_LENENC_TYPE_ERR;
+	}
+
+	return 0;
+}
+
 /**
  * decode a length-encoded integer from a network packet
  *
  * _off is incremented on success 
  *
  * @param packet   the MySQL-packet to decode
- * @param _off     offset in into the packet 
- * @return the decoded number
+ * @param v        destination of the integer
+ * @return 0 on success, non-0 on error 
  *
  */
 int network_mysqld_proto_get_lenenc_int(network_packet *packet, guint64 *v) {
@@ -75,26 +112,24 @@ int network_mysqld_proto_get_lenenc_int(network_packet *packet, guint64 *v) {
 	guint64 ret = 0;
 	unsigned char *bytestream = (unsigned char *)packet->data->str;
 
-	g_return_val_if_fail(off < packet->data->len, -1);
+	if (off >= packet->data->len) return -1;
 	
 	if (bytestream[off] < 251) { /* */
 		ret = bytestream[off];
-	} else if (bytestream[off] == 251) { /* NULL in row-data */
-		ret = bytestream[off];
 	} else if (bytestream[off] == 252) { /* 2 byte length*/
-		g_return_val_if_fail(off + 2 < packet->data->len, -1);
+		if (off + 2 >= packet->data->len) return -1;
 		ret = (bytestream[off + 1] << 0) | 
 			(bytestream[off + 2] << 8) ;
 		off += 2;
 	} else if (bytestream[off] == 253) { /* 3 byte */
-		g_return_val_if_fail(off + 3 < packet->data->len, -1);
+		if (off + 3 >= packet->data->len) return -1;
 		ret = (bytestream[off + 1]   <<  0) | 
 			(bytestream[off + 2] <<  8) |
 			(bytestream[off + 3] << 16);
 
 		off += 3;
 	} else if (bytestream[off] == 254) { /* 8 byte */
-		g_return_val_if_fail(off + 8 < packet->data->len, -1);
+		if (off + 8 >= packet->data->len) return -1;
 		ret = (bytestream[off + 5] << 0) |
 			(bytestream[off + 6] << 8) |
 			(bytestream[off + 7] << 16) |
@@ -114,6 +149,8 @@ int network_mysqld_proto_get_lenenc_int(network_packet *packet, guint64 *v) {
 			G_STRLOC,
 			off, bytestream[off]);
 
+		/* either ERR (255) or NULL (251) */
+
 		return -1;
 	}
 	off += 1;
@@ -131,7 +168,6 @@ int network_mysqld_proto_get_lenenc_int(network_packet *packet, guint64 *v) {
  * a assertion makes sure that we can't skip over the end of the packet 
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
  * @param size   bytes to skip
  *
  */
@@ -147,7 +183,7 @@ int network_mysqld_proto_skip(network_packet *packet, gsize size) {
  * get a fixed-length integer from the network packet 
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
+ * @param v      destination of the integer
  * @param size   byte-len of the integer to decode
  * @return a the decoded integer
  */
@@ -199,8 +235,8 @@ int network_mysqld_proto_get_int_len(network_packet *packet, guint64 *v, gsize s
  * get a 8-bit integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_get_int8(network_packet *packet, guint8 *v) {
@@ -219,8 +255,8 @@ int network_mysqld_proto_get_int8(network_packet *packet, guint8 *v) {
  * get a 8-bit integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_peek_int8(network_packet *packet, guint8 *v) {
@@ -240,8 +276,8 @@ int network_mysqld_proto_peek_int8(network_packet *packet, guint8 *v) {
  * get a 16-bit integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_get_int16(network_packet *packet, guint16 *v) {
@@ -260,8 +296,8 @@ int network_mysqld_proto_get_int16(network_packet *packet, guint16 *v) {
  * get a 16-bit integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_peek_int16(network_packet *packet, guint16 *v) {
@@ -281,8 +317,8 @@ int network_mysqld_proto_peek_int16(network_packet *packet, guint16 *v) {
  * get a 24-bit integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_get_int24(network_packet *packet, guint32 *v) {
@@ -302,8 +338,8 @@ int network_mysqld_proto_get_int24(network_packet *packet, guint32 *v) {
  * get a 32-bit integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_get_int32(network_packet *packet, guint32 *v) {
@@ -320,8 +356,8 @@ int network_mysqld_proto_get_int32(network_packet *packet, guint32 *v) {
  * get a 6-byte integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_get_int48(network_packet *packet, guint64 *v) {
@@ -338,8 +374,8 @@ int network_mysqld_proto_get_int48(network_packet *packet, guint64 *v) {
  * get a 8-byte integer from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return a the decoded integer
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_int_len()
  */
 int network_mysqld_proto_get_int64(network_packet *packet, guint64 *v) {
@@ -350,7 +386,8 @@ int network_mysqld_proto_get_int64(network_packet *packet, guint64 *v) {
  * find a 8-bit integer in the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
+ * @param c      character to find
+ * @param pos    offset into the packet the 'c' was found
  * @return a the decoded integer
  * @see network_mysqld_proto_get_int_len()
  */
@@ -380,8 +417,9 @@ int network_mysqld_proto_find_int8(network_packet *packet, guint8 c, guint *pos)
  * get a string from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
+ * @param s      dest of the string
  * @param len    length of the string
+ * @return       0 on success, non-0 otherwise
  * @return the string (allocated) or NULL of len is 0
  */
 int network_mysqld_proto_get_string_len(network_packet *packet, gchar **s, gsize len) {
@@ -424,8 +462,9 @@ int network_mysqld_proto_get_string_len(network_packet *packet, gchar **s, gsize
  * variable length strings are prefixed with variable-length integer defining the length of the string
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return the string
+ * @param s      destination of the decoded string
+ * @param _len    destination of the length of the decoded string, if len is non-NULL
+ * @return 0 on success, non-0 on error
  * @see network_mysqld_proto_get_string_len(), network_mysqld_proto_get_lenenc_int()
  */
 int network_mysqld_proto_get_lenenc_string(network_packet *packet, gchar **s, guint64 *_len) {
@@ -452,8 +491,8 @@ int network_mysqld_proto_get_lenenc_string(network_packet *packet, gchar **s, gu
  * get a NUL-terminated string from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
- * @return       the string
+ * @param s      dest of the string
+ * @return       0 on success, non-0 otherwise
  * @see network_mysqld_proto_get_string_len()
  */
 int network_mysqld_proto_get_string(network_packet *packet, gchar **s) {
@@ -491,7 +530,6 @@ int network_mysqld_proto_get_string(network_packet *packet, gchar **s) {
  * get a GString from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
  * @param len    bytes to copy
  * @param out    a GString which carries the string
  * @return       0 on success, -1 on error
@@ -520,7 +558,6 @@ int network_mysqld_proto_get_gstring_len(network_packet *packet, gsize len, GStr
  * get a NUL-terminated GString from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
  * @param out    a GString which carries the string
  * @return       a pointer to the string in out
  *
@@ -530,9 +567,11 @@ int network_mysqld_proto_get_gstring(network_packet *packet, GString *out) {
 	guint64 len;
 	int err = 0;
 
-	for (len = 0; packet->offset + len < packet->data->len && *(packet->data->str + packet->offset + len); len++);
+	for (len = 0; packet->offset + len < packet->data->len && *(packet->data->str + packet->offset + len) != '\0'; len++);
 
-	g_assert(*(packet->data->str + packet->offset + len) == '\0'); /* this has to be a \0 */
+	if (packet->offset + len == packet->data->len) { /* havn't found a trailing \0 */
+		return -1;
+	}
 
 	if (len > 0) {
 		g_assert(packet->offset < packet->data->len);
@@ -551,9 +590,8 @@ int network_mysqld_proto_get_gstring(network_packet *packet, GString *out) {
  * get a variable-length GString from the network packet
  *
  * @param packet the MySQL network packet
- * @param _off   offset into the packet
  * @param out    a GString which carries the string
- * @return       a pointer to the string in out
+ * @return       0 on success, non-0 on error
  *
  * @see network_mysqld_proto_get_gstring_len(), network_mysqld_proto_get_lenenc_int()
  */
@@ -644,17 +682,34 @@ void network_mysqld_proto_fielddefs_free(GPtrArray *fields) {
  * @param id      sequence-id of the packet
  * @return 0
  */
-int network_mysqld_proto_set_header(unsigned char *header, size_t length, unsigned char id) {
-	g_assert(length <= PACKET_LEN_MAX);
+int network_mysqld_proto_set_packet_len(GString *_header, guint32 length) {
+	unsigned char *header = (unsigned char *)_header->str;
+
+	g_assert_cmpint(length, <=, PACKET_LEN_MAX);
 
 	header[0] = (length >>  0) & 0xFF;
 	header[1] = (length >>  8) & 0xFF;
 	header[2] = (length >> 16) & 0xFF;
+	
+	return 0;
+}
+
+int network_mysqld_proto_set_packet_id(GString *_header, guint8 id) {
+	unsigned char *header = (unsigned char *)_header->str;
+
 	header[3] = id;
 
 	return 0;
 }
 
+int network_mysqld_proto_append_packet_len(GString *_header, guint32 length) {
+	return network_mysqld_proto_append_int24(_header, length);
+}
+
+int network_mysqld_proto_append_packet_id(GString *_header, guint8 id) {
+	return network_mysqld_proto_append_int8(_header, id);
+}
+
 /**
  * decode the packet length from a packet header
  *
@@ -662,10 +717,26 @@ int network_mysqld_proto_set_header(unsigned char *header, size_t length, unsign
  * @return the packet length
  * @see network_mysqld_proto_set_header()
  */
-size_t network_mysqld_proto_get_header(unsigned char *header) {
+guint32 network_mysqld_proto_get_packet_len(GString *_header) {
+	unsigned char *header = (unsigned char *)_header->str;
+
 	return header[0] | header[1] << 8 | header[2] << 16;
 }
 
+/**
+ * decode the packet length from a packet header
+ *
+ * @param header the first 3 bytes of the network packet
+ * @return the packet length
+ * @see network_mysqld_proto_set_header()
+ */
+guint8 network_mysqld_proto_get_packet_id(GString *_header) {
+	unsigned char *header = (unsigned char *)_header->str;
+
+	return header[3];
+}
+
+
 /**
  * append the variable-length integer to the packet
  *
@@ -828,46 +899,71 @@ int network_mysqld_proto_append_int64(GString *packet, guint64 num) {
 
 
 /**
- * generate the response to the server challenge
+ * hash the password as MySQL 4.1 and later assume
  *
- * SHA1( scramble +
- *       SHA1(SHA1( password )) 
+ *   SHA1( password )
  *
+ * @see network_mysqld_proto_scramble
  */
-int network_mysqld_proto_scramble(GString *response, GString *challenge, const char *password) {
-	int i;
-
-	/* we only have SHA1() in glib 2.16.0 and higher */
-#if GLIB_CHECK_VERSION(2, 16, 0)
+int network_mysqld_proto_password_hash(GString *response, const char *password, gsize password_len) {
 	GChecksum *cs;
-	GString *step1, *step2;
 
 	/* first round: SHA1(password) */
 	cs = g_checksum_new(G_CHECKSUM_SHA1);
 
-	g_checksum_update(cs, (guchar *)password, strlen(password));
+	g_checksum_update(cs, (guchar *)password, password_len);
 
-	step1 = g_string_sized_new(g_checksum_type_get_length(G_CHECKSUM_SHA1));
-
-	step1->len = step1->allocated_len;
-	g_checksum_get_digest(cs, (guchar *)step1->str, &(step1->len));
-
-	g_checksum_free(cs);
-
-	/* second round SHA1(SHA1(password)) */
-	cs = g_checksum_new(G_CHECKSUM_SHA1);
-	
-	step2 = g_string_sized_new(g_checksum_type_get_length(G_CHECKSUM_SHA1));
-	g_checksum_update(cs, (guchar *)step1->str, step1->len);
-	
-	step2->len = step2->allocated_len;
-	g_checksum_get_digest(cs, (guchar *)step2->str, &(step2->len));
+	g_string_set_size(response, g_checksum_type_get_length(G_CHECKSUM_SHA1));
+	response->len = response->allocated_len; /* will be overwritten with the right value in the next step */
+	g_checksum_get_digest(cs, (guchar *)response->str, &(response->len));
 
 	g_checksum_free(cs);
 	
-	/* final round SHA1(challenge + SHA1(SHA1(password))) */
+	return 0;
+}
+
+/**
+ * scramble the hashed password with the challenge
+ *
+ * @param response         dest 
+ * @param challenge        the challenge string as sent by the mysql-server
+ * @param challenge_len    length of the challenge
+ * @param hashed_password  hashed password
+ * @param hashed_password_len length of the hashed password
+ *
+ * @see network_mysqld_proto_password_hash
+ */
+int network_mysqld_proto_password_scramble(GString *response,
+		const char *challenge, gsize challenge_len,
+		const char *hashed_password, gsize hashed_password_len) {
+	int i;
+	GChecksum *cs;
+	GString *step2;
+
+	g_return_val_if_fail(NULL != challenge, -1);
+	g_return_val_if_fail(20 == challenge_len, -1);
+	g_return_val_if_fail(NULL != hashed_password, -1);
+	g_return_val_if_fail(20 == hashed_password_len, -1);
+
+	/**
+	 * we have to run
+	 *
+	 *   XOR( SHA1(password), SHA1(challenge + SHA1(SHA1(password)))
+	 *
+	 * where SHA1(password) is the hashed_password and
+	 *       challenge      is ... challenge
+	 *
+	 *   XOR( hashed_password, SHA1(challenge + SHA1(hashed_password)))
+	 *
+	 */
+
+	/* 1. SHA1(hashed_password) */
+	step2 = g_string_new(NULL);
+	network_mysqld_proto_password_hash(step2, hashed_password, hashed_password_len);
+
+	/* 2. SHA1(challenge + SHA1(hashed_password) */
 	cs = g_checksum_new(G_CHECKSUM_SHA1);
-	g_checksum_update(cs, (guchar *)challenge->str, challenge->len);
+	g_checksum_update(cs, (guchar *)challenge, challenge_len);
 	g_checksum_update(cs, (guchar *)step2->str, step2->len);
 	
 	g_string_set_size(response, g_checksum_type_get_length(G_CHECKSUM_SHA1));
@@ -876,22 +972,126 @@ int network_mysqld_proto_scramble(GString *response, GString *challenge, const c
 	
 	g_checksum_free(cs);
 
-	/* XOR the SHA1(password) with SHA1(challenge + SHA1(SHA1(password))) */
+	/* XOR the hashed_password with SHA1(challenge + SHA1(hashed_password)) */
 	for (i = 0; i < 20; i++) {
-		response->str[i] = (guchar)response->str[i] ^ (guchar)step1->str[i];
+		response->str[i] = (guchar)response->str[i] ^ (guchar)hashed_password[i];
 	}
 
-	g_string_free(step1, TRUE);
 	g_string_free(step2, TRUE);
-#else
-	/* we don't know how to encrypt, so fake it */
-	g_string_set_size(response, 21);
-	for (i = 0; i < 20; i++) response->str[i] = '\0';
-	response->str[20] = '\0';
-#endif
+
 	return 0;
 }
 
+/**
+ * unscramble the auth-response and get the hashed-password
+ *
+ * @param hashed_password  dest of the hashed password
+ * @param challenge        the challenge string as sent by the mysql-server
+ * @param challenge_len    length of the challenge
+ * @param response         auth response as sent by the client 
+ * @param response_len     length of response
+ * @param double_hashed    the double hashed password as stored in the mysql.server table (without * and unhexed)
+ * @param double_hashed_len length of double_hashed
+ *
+ * @see network_mysqld_proto_scramble
+ */
+int network_mysqld_proto_password_unscramble(
+		GString *hashed_password,
+		const char *challenge, gsize challenge_len,
+		const char *response, gsize response_len,
+		const char *double_hashed, gsize double_hashed_len) {
+	int i;
+	GChecksum *cs;
+
+	g_return_val_if_fail(NULL != response, FALSE);
+	g_return_val_if_fail(20 == response_len, FALSE);
+	g_return_val_if_fail(NULL != challenge, FALSE);
+	g_return_val_if_fail(20 == challenge_len, FALSE);
+	g_return_val_if_fail(NULL != double_hashed, FALSE);
+	g_return_val_if_fail(20 == double_hashed_len, FALSE);
+
+	/**
+	 * to check we have to:
+	 *
+	 *   hashed_password = XOR( response, SHA1(challenge + double_hashed))
+	 *   double_hashed == SHA1(hashed_password)
+	 *
+	 * where SHA1(password) is the hashed_password and
+	 *       challenge      is ... challenge
+	 *       response       is the response of the client
+	 *
+	 *   XOR( hashed_password, SHA1(challenge + SHA1(hashed_password)))
+	 *
+	 */
+
+
+	/* 1. SHA1(challenge + double_hashed) */
+	cs = g_checksum_new(G_CHECKSUM_SHA1);
+	g_checksum_update(cs, (guchar *)challenge, challenge_len);
+	g_checksum_update(cs, (guchar *)double_hashed, double_hashed_len);
+	
+	g_string_set_size(hashed_password, g_checksum_type_get_length(G_CHECKSUM_SHA1));
+	hashed_password->len = hashed_password->allocated_len;
+	g_checksum_get_digest(cs, (guchar *)hashed_password->str, &(hashed_password->len));
+	
+	g_checksum_free(cs);
+	
+	/* 2. XOR the response with SHA1(challenge + SHA1(hashed_password)) */
+	for (i = 0; i < 20; i++) {
+		hashed_password->str[i] = (guchar)response[i] ^ (guchar)hashed_password->str[i];
+	}
+
+	return 0;
+}
+
+/**
+ * check if response and challenge match a double-hashed password
+ *
+ * @param challenge        the challenge string as sent by the mysql-server
+ * @param challenge_len    length of the challenge
+ * @param response         auth response as sent by the client 
+ * @param response_len     length of response
+ * @param double_hashed    the double hashed password as stored in the mysql.server table (without * and unhexed)
+ * @param double_hashed_len length of double_hashed
+ *
+ * @see network_mysqld_proto_scramble
+ */
+gboolean network_mysqld_proto_password_check(
+		const char *challenge, gsize challenge_len,
+		const char *response, gsize response_len,
+		const char *double_hashed, gsize double_hashed_len) {
+
+	GString *hashed_password, *step2;
+	gboolean is_same;
+
+	g_return_val_if_fail(NULL != response, FALSE);
+	g_return_val_if_fail(20 == response_len, FALSE);
+	g_return_val_if_fail(NULL != challenge, FALSE);
+	g_return_val_if_fail(20 == challenge_len, FALSE);
+	g_return_val_if_fail(NULL != double_hashed, FALSE);
+	g_return_val_if_fail(20 == double_hashed_len, FALSE);
+
+	hashed_password = g_string_new(NULL);
+
+	network_mysqld_proto_password_unscramble(hashed_password, 
+			challenge, challenge_len,
+			response, response_len,
+			double_hashed, double_hashed_len);
+
+	/* 3. SHA1(hashed_password) */
+	step2 = g_string_new(NULL);
+	network_mysqld_proto_password_hash(step2, S(hashed_password));
+	
+	/* 4. the result of 3 should be the same what we got from the mysql.user table */
+	is_same = strleq(S(step2), double_hashed, double_hashed_len);
+
+	g_string_free(step2, TRUE);
+	g_string_free(hashed_password, TRUE);
+
+	return is_same;
+}
+
+
 network_packet *network_packet_new(void) {
 	network_packet *packet;
 
