@@ -21,119 +21,6 @@
 #include "config.h"
 #endif
 
-/** 
- * @page protocol MySQL Protocol
- *
- * The MySQL Protocol is spilt up into the four phases
- *
- * -# connect
- * -# auth
- * -# query
- * -# and the close state
- *
- * @dot
- * digraph states {
- *   graph [rankdir=LR];
- *   node [fontname=Helvetica, fontsize=10];
- *
- *   connect [ shape=record ];
- *   close [ shape=record ];
- *
- *   subgraph { 
- *     label = "client";
- *     color = black;
- *     rank = same;
- *     node [ style=filled, fillcolor=lightblue ];
- *     connect; 
- *     auth; 
- *     oldauth; 
- *     query; 
- *     local;
- *   }
- *
- *   subgraph { 
- *     label = "server";
- *     rank = same; 
- *     node [ style=filled, fillcolor=orange ];
- *     handshake; 
- *     authres; 
- *     result; 
- *     infile;
- *   }
- *   
- *   subgraph { 
- *     edge [ fontcolor=blue, color=blue, fontsize=10, fontname=Helvetica ];
- *
- *     connect->handshake [ label = "connecting server" ];
- *     auth->authres [ label = "capabilities, password, default-db", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Client_Authentication_Packet" ]; 
- *     oldauth->authres [ label = "scrambled password" ] ; 
- *     query->result [ label = "command (COM_*)", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Command_Packet" ] ;
- *     query->infile [ label = "LOAD DATA INFILE LOCAL" ];
- *     local->result [ label = "file content"];
- *   }
- *
- *   subgraph {
- *     edge [ fontcolor=red, color=red, fontsize=10, fontname=Helvetica ];
- *     handshake->close [ label = "ERR: host denied", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Error_Packet" ];
- *     handshake->auth [ label = "0x10: handshake", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Handshake_Initialization_Packet" ];
- *     authres->oldauth [ label = "EOF: old password reauth" ];
- *     authres->query [ label = "OK: auth done", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#OK_Packet" ];
- *     authres->close [ label = "ERR: auth failed", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Error_Packet" ];
- *     result->query [ label = "result for COM_*", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Result_Set_Header_Packet" ] ;
- *     result->close [ label = "COM_QUIT" ];
- *     result->result [ label = "COM_BINLOG_DUMP" ];
- *     infile->local [ label = "EOF: filename" ];
- *   }
- * }
- * @enddot
- * 
- * Unfolded the sequence diagrams of the different use-cases:
- *
- * -# the client connects to the server and waits for data to return @msc
- *   client, backend;
- *   --- [ label = "connect to backend" ];
- *   client->backend  [ label = "INIT" ];
- * @endmsc
- * -# the auth-phase handles the new SHA1-style passwords and the old scramble() passwords 
- *   -# 4.1+ passwords @msc
- *   client, backend;
- *   --- [ label = "authenticate" ];
- *   backend->client [ label = "HANDSHAKE" ];
- *   client->backend [ label = "AUTH" ];
- *   backend->client [ label = "AUTH_RESULT" ];
- * @endmsc
- *   -# pre-4.1 passwords @msc
- *   client, backend;
- *   --- [ label = "authenticate" ];
- *   backend->client [ label = "HANDSHAKE" ];
- *   client->backend [ label = "AUTH" ];
- *   backend->client [ label = "OLD_PASSWORD_SCRAMBLE" ];
- *   client->backend [ label = "OLD_PASSWORD_AUTH" ];
- *   backend->client [ label = "AUTH_RESULT" ];
- * @endmsc
- * -# the query-phase repeats 
- *   -# COM_QUERY and friends @msc
- *   client, backend;
- *   --- [ label = "query result phase" ];
- *   client->backend [ label = "QUERY" ];
- *   backend->client [ label = "QUERY_RESULT" ];
- * @endmsc
- *   -# COM_QUIT @msc
- *   client, backend;
- *   --- [ label = "query result phase" ];
- *   client->backend [ label = "QUERY" ];
- *   backend->client [ label = "connection close" ];
- * @endmsc
- *   -# COM_BINLOG_DUMP @msc
- *   client, backend;
- *   --- [ label = "query result phase" ];
- *   client->backend [ label = "QUERY" ];
- *   backend->client [ label = "QUERY_RESULT" ];
- *   ... [ label = "more binlog entries" ];
- *   backend->client [ label = "QUERY_RESULT"];
- * @endmsc
- */
-
 #include <sys/types.h>
 
 #ifdef HAVE_SYS_FILIO_H
@@ -179,6 +66,7 @@
 #include "network-mysqld-packet.h"
 #include "network-conn-pool.h"
 #include "chassis-mainloop.h"
+#include "chassis-event-thread.h"
 #include "lua-scope.h"
 #include "glib-ext.h"
 
@@ -221,6 +109,7 @@
  * a handy marco for constant strings 
  */
 #define C(x) x, sizeof(x) - 1
+#define S(x) x->str, x->len
 
 /**
  * call the cleanup callback for the current connection
@@ -305,13 +194,13 @@ network_mysqld_con *network_mysqld_con_init() {
 /**
  * create a connection 
  *
- * @param srv    global context
  * @return       a connection context
  */
 network_mysqld_con *network_mysqld_con_new() {
 	network_mysqld_con *con;
 
 	con = g_new0(network_mysqld_con, 1);
+	con->timestamps = chassis_timestamps_new();
 
 	return con;
 }
@@ -342,6 +231,7 @@ void network_mysqld_con_free(network_mysqld_con *con) {
 	/* we are still in the conns-array */
 
 	g_ptr_array_remove_fast(con->srv->priv->cons, con);
+	chassis_timestamps_free(con->timestamps);
 
 	g_free(con);
 }
@@ -370,18 +260,115 @@ static void dump_str(const char *msg, const unsigned char *s, size_t len) {
 }
 #endif
 
-int network_mysqld_queue_append(network_queue *queue, const char *data, size_t len, int packet_id) {
-	unsigned char header[4];
-	GString *s;
+int network_mysqld_queue_reset(network_socket *sock) {
+	sock->packet_id_is_reset = TRUE;
 
-	network_mysqld_proto_set_header(header, len, packet_id);
+	return 0;
+}
 
-	s = g_string_sized_new(len + 4);
+/**
+ * synchronize the packet-ids of two network-sockets 
+ */
+int network_mysqld_queue_sync(network_socket *dst, network_socket *src) {
+	g_assert_cmpint(src->packet_id_is_reset, ==, FALSE);
 
-	g_string_append_len(s, (gchar *)header, 4);
-	g_string_append_len(s, data, len);
+	if (dst->packet_id_is_reset == FALSE) {
+		/* this shouldn't really happen */
+	}
 
-	network_queue_append(queue, s);
+	dst->last_packet_id = src->last_packet_id - 1;
+
+	return 0;
+}
+
+/**
+ * appends a raw MySQL packet to the queue 
+ *
+ * the packet is append the queue directly and shouldn't be used by the caller afterwards anymore
+ * and has to by in the MySQL Packet format
+ *
+ */
+int network_mysqld_queue_append_raw(network_socket *sock, network_queue *queue, GString *data) {
+	guint32 packet_len;
+	guint8  packet_id;
+
+	/* check that the length header is valid */
+	if (queue != sock->send_queue &&
+	    queue != sock->recv_queue) {
+		g_critical("%s: queue = %p doesn't belong to sock %p",
+				G_STRLOC,
+				(void *)queue,
+				(void *)sock);
+		return -1;
+	}
+
+	g_assert_cmpint(data->len, >=, 4);
+
+	packet_len = network_mysqld_proto_get_packet_len(data);
+	packet_id  = network_mysqld_proto_get_packet_id(data);
+
+	g_assert_cmpint(packet_len, ==, data->len - 4);
+
+	if (sock->packet_id_is_reset) {
+		/* the ->last_packet_id is undefined, accept what we get */
+		sock->last_packet_id = packet_id;
+		sock->packet_id_is_reset = FALSE;
+	} else if (packet_id != (guint8)(sock->last_packet_id + 1)) {
+		sock->last_packet_id++;
+#if 0
+		g_critical("%s: packet-id %d doesn't match for socket's last packet %d, patching it",
+				G_STRLOC,
+				packet_id,
+				sock->last_packet_id);
+#endif
+		network_mysqld_proto_set_packet_id(data, sock->last_packet_id);
+	} else {
+		sock->last_packet_id++;
+	}
+
+	network_queue_append(queue, data);
+
+	return 0;
+}
+
+/**
+ * appends a payload to the queue
+ *
+ * the packet is copied and prepened with the mysql packet header before it is appended to the queue
+ * if neccesary the payload is spread over multiple mysql packets
+ */
+int network_mysqld_queue_append(network_socket *sock, network_queue *queue, const char *data, size_t packet_len) {
+	gsize packet_offset = 0;
+
+	do {
+		GString *s;
+		gsize cur_packet_len = MIN(packet_len, PACKET_LEN_MAX);
+
+		s = g_string_sized_new(packet_len + 4);
+
+		if (sock->packet_id_is_reset) {
+			sock->packet_id_is_reset = FALSE;
+			sock->last_packet_id = 0xff; /** the ++last_packet_id will make sure we send a 0 */
+		}
+
+		network_mysqld_proto_append_packet_len(s, cur_packet_len);
+		network_mysqld_proto_append_packet_id(s, ++sock->last_packet_id);
+		g_string_append_len(s, data + packet_offset, cur_packet_len);
+
+		network_queue_append(queue, s);
+
+		if (packet_len == PACKET_LEN_MAX) {
+			s = g_string_sized_new(4);
+
+			network_mysqld_proto_append_packet_len(s, 0);
+			network_mysqld_proto_append_packet_id(s, ++sock->last_packet_id);
+
+			network_queue_append(queue, s);
+		}
+
+		packet_len -= cur_packet_len;
+		packet_offset += cur_packet_len;
+	} while (packet_len > 0);
 
 	return 0;
 }
@@ -411,7 +398,8 @@ int network_mysqld_con_send_ok_full(network_socket *con, guint64 affected_rows,
 
 	network_mysqld_proto_append_ok_packet(packet, ok_packet);
 	
-	network_mysqld_queue_append(con->send_queue, packet->str, packet->len, con->packet_id);
+	network_mysqld_queue_append(con, con->send_queue, S(packet));
+	network_mysqld_queue_reset(con);
 
 	g_string_free(packet, TRUE);
 	network_mysqld_ok_packet_free(ok_packet);
@@ -459,7 +447,8 @@ int network_mysqld_con_send_error_full(network_socket *con, const char *errmsg,
 
 	network_mysqld_proto_append_err_packet(packet, err_packet);
 
-	network_mysqld_queue_append(con->send_queue, packet->str, packet->len, con->packet_id);
+	network_mysqld_queue_append(con, con->send_queue, S(packet));
+	network_mysqld_queue_reset(con);
 
 	network_mysqld_err_packet_free(err_packet);
 	g_string_free(packet, TRUE);
@@ -482,6 +471,56 @@ int network_mysqld_con_send_error(network_socket *con, const char *errmsg, gsize
 	return network_mysqld_con_send_error_full(con, errmsg, errmsg_len, ER_UNKNOWN_ERROR, NULL);
 }
 
+/**
+ * get a full packet from the raw queue and move it to the packet queue 
+ */
+network_socket_retval_t network_mysqld_con_get_packet(chassis G_GNUC_UNUSED*chas, network_socket *con) {
+	GString *packet = NULL;
+	GString header;
+	char header_str[NET_HEADER_SIZE + 1] = "";
+	guint32 packet_len;
+	guint8  packet_id;
+
+	/** 
+	 * read the packet header (4 bytes)
+	 */
+	header.str = header_str;
+	header.allocated_len = sizeof(header_str);
+	header.len = 0;
+
+	/* read the packet len if the leading packet */
+	if (!network_queue_peek_string(con->recv_queue_raw, NET_HEADER_SIZE, &header)) {
+		/* too small */
+
+		return NETWORK_SOCKET_WAIT_FOR_EVENT;
+	}
+
+	packet_len = network_mysqld_proto_get_packet_len(&header);
+	packet_id  = network_mysqld_proto_get_packet_id(&header);
+
+	/* move the packet from the raw queue to the recv-queue */
+	if ((packet = network_queue_pop_string(con->recv_queue_raw, packet_len + NET_HEADER_SIZE, NULL))) {
+		if (con->packet_id_is_reset) {
+			con->last_packet_id = packet_id;
+			con->packet_id_is_reset = FALSE;
+		} else if (packet_id != (guint8)(con->last_packet_id + 1)) {
+			g_critical("%s: received packet-id %d, but expected %d ... out of sync.",
+					G_STRLOC,
+					packet_id,
+					con->last_packet_id + 1);
+			return NETWORK_SOCKET_ERROR;
+		} else {
+			con->last_packet_id = packet_id;
+		}
+	
+		network_queue_append(con->recv_queue, packet);
+	} else {
+		return NETWORK_SOCKET_WAIT_FOR_EVENT;
+	}
+
+	return NETWORK_SOCKET_SUCCESS;
+}
+
 /**
  * read a MySQL packet from the socket
  *
@@ -489,9 +528,6 @@ int network_mysqld_con_send_error(network_socket *con, const char *errmsg, gsize
  * with packet-header and everything 
  */
 network_socket_retval_t network_mysqld_read(chassis G_GNUC_UNUSED*chas, network_socket *con) {
-	GString *packet = NULL;
-
-	/* check if the recv_queue is clean up to now */
 	switch (network_socket_read(con)) {
 	case NETWORK_SOCKET_WAIT_FOR_EVENT:
 		return NETWORK_SOCKET_WAIT_FOR_EVENT;
@@ -504,36 +540,7 @@ network_socket_retval_t network_mysqld_read(chassis G_GNUC_UNUSED*chas, network_
 		break;
 	}
 
-	/** 
-	 * read the packet header (4 bytes)
-	 */
-	if (con->packet_len == PACKET_LEN_UNSET) {
-		GString header;
-		char header_str[NET_HEADER_SIZE + 1] = "";
-
-		header.str = header_str;
-		header.allocated_len = sizeof(header_str);
-		header.len = 0;
-
-		if (!network_queue_peek_string(con->recv_queue_raw, NET_HEADER_SIZE, &header)) {
-			/* too small */
-
-			return NETWORK_SOCKET_WAIT_FOR_EVENT;
-		}
-
-		con->packet_len = network_mysqld_proto_get_header((unsigned char *)(header_str));
-		con->packet_id  = (unsigned char)(header_str[3]); /* packet-id if the next packet */
-	}
-/* TODO: convert to DTrace probe
-	g_debug("[%s] read packet id %d of length %d", G_STRLOC, con->packet_id, con->packet_len + NET_HEADER_SIZE); */
-	/* move the packet from the raw queue to the recv-queue */
-	if ((packet = network_queue_pop_string(con->recv_queue_raw, con->packet_len + NET_HEADER_SIZE, NULL))) {
-		network_queue_append(con->recv_queue, packet);
-	} else {
-		return NETWORK_SOCKET_WAIT_FOR_EVENT;
-	}
-
-	return NETWORK_SOCKET_SUCCESS;
+	return network_mysqld_con_get_packet(chas, con);
 }
 
 network_socket_retval_t network_mysqld_write(chassis G_GNUC_UNUSED*chas, network_socket *con) {
@@ -650,11 +657,8 @@ network_socket_retval_t plugin_call(chassis *srv, network_mysqld_con *con, int s
 		packet = chunk->data;
 
 		/* we aren't finished yet */
-		if (packet->len != recv_sock->packet_len + NET_HEADER_SIZE) return NETWORK_SOCKET_SUCCESS;
-
 		network_queue_append(send_sock->send_queue, packet);
 
-		recv_sock->packet_len = PACKET_LEN_UNSET;
 		g_queue_delete_link(recv_sock->recv_queue->chunks, chunk);
 
 		/**
@@ -734,10 +738,34 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 	if (events == EV_READ) {
 		int b = -1;
 
+		/**
+		 * check how much data there is to read
+		 *
+		 * ioctl()
+		 * - returns 0 if connection is closed
+		 * - or -1 and ECONNRESET on solaris
+		 *   or -1 and EPIPE on HP/UX
+		 */
 		if (ioctl(event_fd, FIONREAD, &b)) {
-			g_critical("ioctl(%d, FIONREAD, ...) failed: %s", event_fd, g_strerror(errno));
+			switch (errno) {
+			case E_NET_CONNRESET: /* solaris */
+			case EPIPE: /* hp/ux */
+				if (con->client && event_fd == con->client->fd) {
+					/* the client closed the connection, let's keep the server side open */
+					con->state = CON_STATE_CLOSE_CLIENT;
+				} else if (con->server && event_fd == con->server->fd && con->com_quit_seen) {
+					con->state = CON_STATE_CLOSE_SERVER;
+				} else {
+					/* server side closed on use, oops, close both sides */
+					con->state = CON_STATE_ERROR;
+				}
+				break;
+			default:
+				g_critical("ioctl(%d, FIONREAD, ...) failed: %s", event_fd, g_strerror(errno));
 
-			con->state = CON_STATE_ERROR;
+				con->state = CON_STATE_ERROR;
+				break;
+			}
 		} else if (b != 0) {
 			if (con->client && event_fd == con->client->fd) {
 				con->client->to_read = b;
@@ -746,7 +774,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			} else {
 				g_error("%s.%d: neither nor", __FILE__, __LINE__);
 			}
-		} else {
+		} else { /* Linux */
 			if (con->client && event_fd == con->client->fd) {
 				/* the client closed the connection, let's keep the server side open */
 				con->state = CON_STATE_CLOSE_CLIENT;
@@ -761,12 +789,17 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 
 #define WAIT_FOR_EVENT(ev_struct, ev_type, timeout) \
 	event_set(&(ev_struct->event), ev_struct->fd, ev_type, network_mysqld_con_handle, user_data); \
-	event_base_set(srv->event_base, &(ev_struct->event));\
-	event_add(&(ev_struct->event), timeout);
+	chassis_event_add(srv, &(ev_struct->event)); 
 
 	/**
 	 * loop on the same connection as long as we don't end up in a stable state
 	 */
+
+	if (event_fd != -1) {
+		NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::done");
+	} else {
+		NETWORK_MYSQLD_CON_TRACK_TIME(con, "con_handle_start");
+	}
 	do {
 		ostate = con->state;
 		MYSQLPROXY_STATE_CHANGE(event_fd, events, con->state);
@@ -794,7 +827,51 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			/* FIXME: this comment has nothing to do with reality...
 			 * the server connection is still fine, 
 			 * let's keep it open for reuse */
+
 			plugin_call_cleanup(srv, con);
+
+			/* dump the timestamps of this connection */
+			if (srv->log->min_lvl == G_LOG_LEVEL_DEBUG) {
+				GList *node;
+				guint64 abs_usec = 0;
+				guint64 wait_event_usec = 0;
+				guint64 lua_usec = 0;
+
+				for (node = con->timestamps->timestamps->head; node; node = node->next) {
+					chassis_timestamp_t *prev = node->prev ? node->prev->data : NULL;
+					chassis_timestamp_t *cur = node->data;
+					guint64 rel_usec = prev ? cur->usec - prev->usec: 0;
+					guint64 rel_cycles = prev ? cur->cycles - prev->cycles: 0;
+
+					abs_usec += rel_usec;
+
+					g_debug("%-35s usec=%8"G_GUINT64_FORMAT", cycles=%8"G_GUINT64_FORMAT", abs-usec=%8"G_GUINT64_FORMAT" (%s:%d)",
+							cur->name,
+							rel_usec,
+							rel_cycles,
+							abs_usec,
+							cur->filename, cur->line
+					       );
+
+					if (strstr(cur->name, "leave_lua")) {
+						lua_usec += rel_usec;
+					} else if (strstr(cur->name, "wait_for_event::done")) {
+						wait_event_usec += rel_usec;
+					}
+				}
+
+				g_debug("%-35s usec=%8"G_GUINT64_FORMAT"",
+						"abs wait-for-event::done",
+						wait_event_usec
+				       );
+				g_debug("%-35s usec=%8"G_GUINT64_FORMAT"",
+						"abs lua-exec::done",
+						lua_usec
+				       );
+
+
+			}
+
 			network_mysqld_con_free(con);
 
 			con = NULL;
@@ -842,6 +919,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 					 * we have a server connection waiting to begin writable
 					 */
 					WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+					NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::connect_server");
 					return;
 				} else {
 					/* try to get a connection to another backend,
@@ -885,6 +963,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				/* call us again when you have a event */
 				WAIT_FOR_EVENT(con->server, EV_READ, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_handshake");
 
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -915,20 +994,6 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				con->state = CON_STATE_ERROR;
 				break;
 			}
-
-			if (recv_sock->challenge &&
-			    recv_sock->challenge->server_version > 50113 && recv_sock->challenge->server_version < 50118) {
-				/**
-				 * Bug #25371
-				 *
-				 * COM_CHANGE_USER returns 2 ERR packets instead of one
-				 *
-				 * we can auto-correct the issue if needed and remove the second packet
-				 * Some clients handle this issue and expect a double ERR packet.
-				 */
-
-				con->state = CON_STATE_ERROR;
-			}
 	
 			break; }
 		case CON_STATE_SEND_HANDSHAKE: 
@@ -939,6 +1004,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_handshake");
 				
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -975,6 +1041,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->client, EV_READ, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_auth");
 
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1006,6 +1073,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_auth");
 
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1042,6 +1110,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->server, EV_READ, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_auth_result");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
 			case NETWORK_SOCKET_ERROR:
@@ -1083,6 +1152,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_auth_result");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
 			case NETWORK_SOCKET_ERROR:
@@ -1111,6 +1181,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->client, EV_READ, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_auth_old_password");
 
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1139,6 +1210,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_auth_old_password");
 
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1163,23 +1235,56 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 
 		case CON_STATE_READ_QUERY: {
 			network_socket *recv_sock;
+			network_packet last_packet;
+
 			recv_sock = con->client;
 
 			g_assert(events == 0 || event_fd == recv_sock->fd);
 
-			switch (network_mysqld_read(srv, recv_sock)) {
-			case NETWORK_SOCKET_SUCCESS:
-				break;
-			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_READ, NULL);
-				return;
-			case NETWORK_SOCKET_ERROR_RETRY:
-			case NETWORK_SOCKET_ERROR:
-				g_critical("%s.%d: network_mysqld_read(CON_STATE_READ_QUERY) returned an error", __FILE__, __LINE__);
-				con->state = CON_STATE_ERROR;
-				return;
+			do { 
+				switch (network_mysqld_read(srv, recv_sock)) {
+				case NETWORK_SOCKET_SUCCESS:
+					break;
+				case NETWORK_SOCKET_WAIT_FOR_EVENT:
+					WAIT_FOR_EVENT(con->client, EV_READ, NULL);
+					NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_query");
+					return;
+				case NETWORK_SOCKET_ERROR_RETRY:
+				case NETWORK_SOCKET_ERROR:
+					g_critical("%s.%d: network_mysqld_read(CON_STATE_READ_QUERY) returned an error", __FILE__, __LINE__);
+					con->state = CON_STATE_ERROR;
+					return;
+				}
+				if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
+
+				last_packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
+			} while (last_packet.data->len == PACKET_LEN_MAX + NET_HEADER_SIZE);
+
+			if (con->server &&
+			    con->server->challenge &&
+			    con->server->challenge->server_version > 50113 && con->server->challenge->server_version < 50118) {
+				/**
+				 * Bug #25371
+				 *
+				 * COM_CHANGE_USER returns 2 ERR packets instead of one
+				 *
+				 * we can auto-correct the issue if needed and remove the second packet
+				 * Some clients handle this issue and expect a double ERR packet.
+				 */
+				network_packet packet;
+				guint8 com;
+
+				packet.data = g_queue_peek_head(recv_sock->recv_queue->chunks);
+				packet.offset = 0;
+
+				if (0 == network_mysqld_proto_skip_network_header(&packet) &&
+				    0 == network_mysqld_proto_get_int8(&packet, &com)  &&
+				    com == COM_CHANGE_USER) {
+					network_mysqld_con_send_error(con->client, C("COM_CHANGE_USER is broken on 5.1.14-.17, please upgrade the MySQL Server"));
+					con->state = CON_STATE_SEND_QUERY_RESULT;
+					break;
+				}
 			}
-			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
 
 			switch (plugin_call(srv, con, con->state)) {
 			case NETWORK_SOCKET_SUCCESS:
@@ -1195,7 +1300,13 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 
 			break; }
 		case CON_STATE_SEND_QUERY:
-			/* send the query to the server */
+			/* send the query to the server
+			 *
+			 * this state will loop until all the packets from the send-queue are flushed 
+			 *
+			 * we have to handle LOAD DATA INFILE LOCAL which can only happen for COM_QUERY packets
+			 */
+
 			if (con->server->send_queue->offset == 0) {
 				/* only parse the packets once */
 				network_packet packet;
@@ -1207,23 +1318,12 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				packet.data = chunk->data;
 				packet.offset = 0;
 
-				/* only parse once and don't care about the blocking read */
+				/*  */
 				if (con->parse.command == COM_QUERY) {
 					network_mysqld_com_query_result_track_state(&packet, con->parse.data);
-				} else if (con->is_overlong_packet) {
-					/* the last packet was a over-long packet
-					 * this is the same command, just more data */
-	
-					if (con->parse.len != PACKET_LEN_MAX) {
-						con->is_overlong_packet = 0;
-					}
 				} else {
 					con->parse.command = packet.data->str[4];
 	
-					if (con->parse.len == PACKET_LEN_MAX) {
-						con->is_overlong_packet = 1;
-					}
-		
 					/* init the parser for the commands */
 					switch (con->parse.command) {
 					case COM_QUERY:
@@ -1256,11 +1356,12 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				}
 			}
 	
-			switch (network_socket_write(con->server, 1)) {
+			switch (network_mysqld_write(srv, con->server)) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_query");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
 			case NETWORK_SOCKET_ERROR:
@@ -1275,11 +1376,12 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			
 			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
 
+#if 0
 			/* in case we are waiting for LOAD DATA LOCAL INFILE data on this connection,
 			 we need to read the sentinel zero-length packet, too. Otherwise we will block
 			 this connection. Bug#37404
 			 */
-			if (con->is_overlong_packet || con->in_load_data_local_state) {
+			if (con->in_load_data_local_state) {
 				/* the last packet of LOAD DATA LOCAL INFILE is zero-length, signalling "no more data following" */
 				if (con->parse.len == 0) {
 					con->state = CON_STATE_READ_QUERY_RESULT;
@@ -1289,12 +1391,15 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				}
 				break;
 			}
+#endif
 
 			/* some statements don't have a server response */
 			switch (con->parse.command) {
 			case COM_STMT_SEND_LONG_DATA: /* not acked */
 			case COM_STMT_CLOSE:
 				con->state = CON_STATE_READ_QUERY;
+				if (con->client) network_mysqld_queue_reset(con->client);
+				if (con->server) network_mysqld_queue_reset(con->server);
 				break;
 			case COM_QUERY:
 				if (network_mysqld_com_query_result_is_load_data(con->parse.data)) {
@@ -1326,6 +1431,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 					break;
 				case NETWORK_SOCKET_WAIT_FOR_EVENT:
 					WAIT_FOR_EVENT(con->server, EV_READ, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_query_result");
 					return;
 				case NETWORK_SOCKET_ERROR_RETRY:
 				case NETWORK_SOCKET_ERROR:
@@ -1367,6 +1473,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_query_result");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
 			case NETWORK_SOCKET_ERROR:
@@ -1408,6 +1515,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
 				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_error");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
 			case NETWORK_SOCKET_ERROR:
@@ -1425,6 +1533,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 		event_fd = -1;
 		events   = 0;
 	} while (ostate != con->state);
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "con_handle_end");
 
 	return;
 }
@@ -1454,6 +1563,8 @@ void network_mysqld_con_accept(int G_GNUC_UNUSED event_fd, short events, void *u
 	client_con = network_mysqld_con_new();
 	client_con->client = client;
 
+	NETWORK_MYSQLD_CON_TRACK_TIME(client_con, "accept");
+
 	network_mysqld_add_connection(listen_con->srv, client_con);
 
 	
@@ -1506,7 +1617,7 @@ int network_mysqld_con_send_resultset(network_socket *con, GPtrArray *fields, GP
 	 */
 
 	network_mysqld_proto_append_lenenc_int(s, fields->len); /* the field-count */
-	network_mysqld_queue_append(con->send_queue, s->str, s->len, con->packet_id++);
+	network_mysqld_queue_append(con, con->send_queue, S(s));
 
 	for (i = 0; i < fields->len; i++) {
 		MYSQL_FIELD *field = fields->pdata[i];
@@ -1535,7 +1646,7 @@ int network_mysqld_con_send_resultset(network_socket *con, GPtrArray *fields, GP
 		/* this is in the docs, but not on the network */
 		network_mysqld_proto_append_lenenc_string(s, field->def);         /* default-value */
 #endif
-		network_mysqld_queue_append(con->send_queue, s->str, s->len, con->packet_id++);
+		network_mysqld_queue_append(con, con->send_queue, S(s));
 	}
 
 	g_string_truncate(s, 0);
@@ -1545,7 +1656,7 @@ int network_mysqld_con_send_resultset(network_socket *con, GPtrArray *fields, GP
 	g_string_append_len(s, "\x00\x00", 2); /* warning count */
 	g_string_append_len(s, "\x02\x00", 2); /* flags */
 	
-	network_mysqld_queue_append(con->send_queue, s->str, s->len, con->packet_id++);
+	network_mysqld_queue_append(con, con->send_queue, S(s));
 
 	for (i = 0; i < rows->len; i++) {
 		GPtrArray *row = rows->pdata[i];
@@ -1555,7 +1666,8 @@ int network_mysqld_con_send_resultset(network_socket *con, GPtrArray *fields, GP
 		for (j = 0; j < row->len; j++) {
 			network_mysqld_proto_append_lenenc_string(s, row->pdata[j]);
 		}
-		network_mysqld_queue_append(con->send_queue, s->str, s->len, con->packet_id++);
+
+		network_mysqld_queue_append(con, con->send_queue, S(s));
 	}
 
 	g_string_truncate(s, 0);
@@ -1565,7 +1677,8 @@ int network_mysqld_con_send_resultset(network_socket *con, GPtrArray *fields, GP
 	g_string_append_len(s, "\x00\x00", 2); /* warning count */
 	g_string_append_len(s, "\x02\x00", 2); /* flags */
 
-	network_mysqld_queue_append(con->send_queue, s->str, s->len, con->packet_id++);
+	network_mysqld_queue_append(con, con->send_queue, S(s));
+	network_mysqld_queue_reset(con);
 
 	g_string_free(s, TRUE);
 
