@@ -88,126 +88,6 @@
 #include "network-mysqld-proto.h"
 #include "network-mysqld-packet.h"
 
-#ifndef DISABLE_DEPRECATED_DECL
-network_queue *network_queue_init() {
-	return network_queue_new();
-}
-#endif
-
-network_queue *network_queue_new() {
-	network_queue *queue;
-
-	queue = g_new0(network_queue, 1);
-
-	queue->chunks = g_queue_new();
-	
-	return queue;
-}
-
-void network_queue_free(network_queue *queue) {
-	GString *packet;
-
-	if (!queue) return;
-
-	while ((packet = g_queue_pop_head(queue->chunks))) g_string_free(packet, TRUE);
-
-	g_queue_free(queue->chunks);
-
-	g_free(queue);
-}
-
-int network_queue_append(network_queue *queue, GString *s) {
-	queue->len += s->len;
-
-	g_queue_push_tail(queue->chunks, s);
-
-	return 0;
-}
-
-/**
- * get a string from the head of the queue and leave the queue unchanged 
- *
- * @param  peek_len bytes to collect
- * @param  dest 
- * @return NULL if not enough data
- *         if dest is not NULL, dest, otherwise a new GString containing the data
- */
-GString *network_queue_peek_string(network_queue *queue, gsize peek_len, GString *dest) {
-	gsize we_want = peek_len;
-	GList *node;
-
-/* TODO: convert to DTrace probe
-	g_debug("[%s] looking for %d bytes, queue has %d", G_STRLOC, peek_len, queue->len); */
-	if (queue->len < peek_len) {
-		return NULL;
-	}
-
-	if (!dest) {
-		/* no define */
-		dest = g_string_sized_new(peek_len);
-	}
-
-	g_assert_cmpint(dest->allocated_len, >, peek_len);
-
-	for (node = queue->chunks->head; node && we_want; node = node->next) {
-		GString *chunk = node->data;
-
-		if (node == queue->chunks->head) {
-			gsize we_have = we_want < (chunk->len - queue->offset) ? we_want : (chunk->len - queue->offset);
-
-			g_string_append_len(dest, chunk->str + queue->offset, we_have);
-			
-			we_want -= we_have;
-		} else {
-			gsize we_have = we_want < chunk->len ? we_want : chunk->len;
-			
-			g_string_append_len(dest, chunk->str, we_have);
-
-			we_want -= we_have;
-		}
-	}
-
-	return dest;
-}
-
-/**
- * get a string from the head of the queue and remove the chunks from the queue 
- */
-GString *network_queue_pop_string(network_queue *queue, gsize steal_len, GString *dest) {
-	gsize we_want = steal_len;
-	GString *chunk;
-
-	if (queue->len < steal_len) {
-		return NULL;
-	}
-
-	if (!dest) {
-		dest = g_string_sized_new(steal_len);
-	}
-	
-	g_assert_cmpint(dest->allocated_len, >, steal_len);
-
-	while ((chunk = g_queue_peek_head(queue->chunks))) {
-		gsize we_have = we_want < (chunk->len - queue->offset) ? we_want : (chunk->len - queue->offset);
-
-		g_string_append_len(dest, chunk->str + queue->offset, we_have);
-
-		queue->offset += we_have;
-		queue->len    -= we_have;
-		we_want -= we_have;
-
-		if (chunk->len == queue->offset) {
-			/* the chunk is done, remove it */
-			g_string_free(g_queue_pop_head(queue->chunks), TRUE);
-			queue->offset = 0;
-		} else {
-			break;
-		}
-	}
-
-	return dest;
-}
-
 #ifndef DISABLE_DEPRECATED_DECL
 network_socket *network_socket_init() {
 	return network_socket_new();
@@ -223,10 +103,10 @@ network_socket *network_socket_new() {
 	s->recv_queue = network_queue_new();
 	s->recv_queue_raw = network_queue_new();
 
-	s->packet_len = PACKET_LEN_UNSET;
-
 	s->default_db = g_string_new(NULL);
 	s->fd           = -1;
+	s->socket_type  = SOCK_STREAM; /* let's default to TCP */
+	s->packet_id_is_reset = TRUE;
 
 	s->src = network_address_new();
 	s->dst = network_address_new();
@@ -261,8 +141,8 @@ void network_socket_free(network_socket *s) {
 /**
  * portable 'set non-blocking io'
  *
- * @param fd      socket-fd
- * @return        0
+ * @param sock    a socket
+ * @return        NETWORK_SOCKET_SUCCESS on success, NETWORK_SOCKET_ERROR on error
  */
 network_socket_retval_t network_socket_set_non_blocking(network_socket *sock) {
 	int ret;
@@ -291,15 +171,14 @@ network_socket_retval_t network_socket_set_non_blocking(network_socket *sock) {
  *
  * event handler for listening connections
  *
- * @param event_fd     fd on which the event was fired
- * @param events       the event that was fired
- * @param user_data    the listening connection handle
+ * @param srv    a listening socket 
  * 
  */
 network_socket *network_socket_accept(network_socket *srv) {
 	network_socket *client;
 
 	g_return_val_if_fail(srv, NULL);
+	g_return_val_if_fail(srv->socket_type == SOCK_STREAM, NULL); /* accept() only works on stream sockets */
 
 	client = network_socket_new();
 
@@ -327,8 +206,11 @@ network_socket *network_socket_accept(network_socket *srv) {
 }
 
 static network_socket_retval_t network_socket_connect_setopts(network_socket *sock) {
+#ifdef WIN32
+	char val = 1;	/* Win32 setsockopt wants a const char* instead of the UNIX void*...*/
+#else
 	int val = 1;
-
+#endif
 	/**
 	 * set the same options as the mysql client 
 	 */
@@ -365,19 +247,22 @@ static network_socket_retval_t network_socket_connect_setopts(network_socket *so
  */
 network_socket_retval_t network_socket_connect_finish(network_socket *sock) {
 	int so_error = 0;
-	socklen_t so_error_len = sizeof(so_error);
+	network_socklen_t so_error_len = sizeof(so_error);
 
 	/**
 	 * we might get called a 2nd time after a connect() == EINPROGRESS
 	 */
-	if (getsockopt(sock->fd, SOL_SOCKET, SO_ERROR, &so_error, &so_error_len)) {
 #ifdef _WIN32
+	/* need to cast to get rid of the compiler warning. otherwise identical to the UNIX version below. */
+	if (getsockopt(sock->fd, SOL_SOCKET, SO_ERROR, (char*)&so_error, &so_error_len)) {
 		errno = WSAGetLastError();
+#else
+	if (getsockopt(sock->fd, SOL_SOCKET, SO_ERROR, &so_error, &so_error_len)) {
 #endif
 		/* getsockopt failed */
-		g_critical("%s: getsockopt(%s) failed: %s", 
+		g_critical("%s: getsockopt(%s) failed: %s (%d)", 
 				G_STRLOC,
-				sock->dst->name->str, g_strerror(errno));
+				sock->dst->name->str, g_strerror(errno), errno);
 		return NETWORK_SOCKET_ERROR;
 	}
 
@@ -398,21 +283,22 @@ network_socket_retval_t network_socket_connect_finish(network_socket *sock) {
  *
  * the sock->addr has to be set before 
  * 
- * @param con    a socket 
- * @return       0 on connected, -1 on error, -2 for try again
+ * @param sock    a socket 
+ * @return        NETWORK_SOCKET_SUCCESS on connected, NETWORK_SOCKET_ERROR on error, NETWORK_SOCKET_ERROR_RETRY for try again
  * @see network_address_set_address()
  */
 network_socket_retval_t network_socket_connect(network_socket *sock) {
 	g_return_val_if_fail(sock->dst, NETWORK_SOCKET_ERROR); /* our _new() allocated it already */
 	g_return_val_if_fail(sock->dst->name->len, NETWORK_SOCKET_ERROR); /* we want to use the ->name in the error-msgs */
 	g_return_val_if_fail(sock->fd < 0, NETWORK_SOCKET_ERROR); /* we already have a valid fd, we don't want to leak it */
+	g_return_val_if_fail(sock->socket_type == SOCK_STREAM, NETWORK_SOCKET_ERROR);
 
 	/**
 	 * create a socket for the requested address
 	 *
 	 * if the dst->addr isn't set yet, socket() will fail with unsupported type
 	 */
-	if (-1 == (sock->fd = socket(sock->dst->addr.common.sa_family, SOCK_STREAM, 0))) {
+	if (-1 == (sock->fd = socket(sock->dst->addr.common.sa_family, sock->socket_type, 0))) {
 #ifdef _WIN32
 		errno = WSAGetLastError();
 #endif
@@ -465,52 +351,80 @@ network_socket_retval_t network_socket_connect(network_socket *sock) {
  * @see network_address_set_address()
  */
 network_socket_retval_t network_socket_bind(network_socket * con) {
+#ifdef WIN32
+	char val = 1;	/* Win32 setsockopt wants a const char* instead of the UNIX void*...*/
+#else
 	int val = 1;
-
-	g_return_val_if_fail(con->dst, NETWORK_SOCKET_ERROR);
-	g_return_val_if_fail(con->dst->name->len > 0, NETWORK_SOCKET_ERROR);
+#endif
 	g_return_val_if_fail(con->fd < 0, NETWORK_SOCKET_ERROR); /* socket is already bound */
+	g_return_val_if_fail((con->socket_type == SOCK_DGRAM) || (con->socket_type == SOCK_STREAM), NETWORK_SOCKET_ERROR);
 
-	if (-1 == (con->fd = socket(con->dst->addr.common.sa_family, SOCK_STREAM, 0))) {
-		g_critical("%s: socket(%s) failed: %s (%d)", 
-				G_STRLOC,
-				con->dst->name->str,
-				g_strerror(errno), errno);
-		return NETWORK_SOCKET_ERROR;
-	}
+	if (con->socket_type == SOCK_STREAM) {
+		g_return_val_if_fail(con->dst, NETWORK_SOCKET_ERROR);
+		g_return_val_if_fail(con->dst->name->len > 0, NETWORK_SOCKET_ERROR);
 
-	if (con->dst->addr.common.sa_family == AF_INET || 
-	    con->dst->addr.common.sa_family == AF_INET6) {
-		if (0 != setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val))) {
-			g_critical("%s: setsockopt(%s, IPPROTO_TCP, TCP_NODELAY) failed: %s (%d)", 
+		if (-1 == (con->fd = socket(con->dst->addr.common.sa_family, con->socket_type, 0))) {
+			g_critical("%s: socket(%s) failed: %s (%d)", 
 					G_STRLOC,
 					con->dst->name->str,
 					g_strerror(errno), errno);
 			return NETWORK_SOCKET_ERROR;
 		}
-		if (0 != setsockopt(con->fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
-			g_critical("%s: setsockopt(%s, SOL_SOCKET, SO_REUSEADDR) failed: %s (%d)", 
+
+		if (con->dst->addr.common.sa_family == AF_INET || 
+		    con->dst->addr.common.sa_family == AF_INET6) {
+			if (0 != setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val))) {
+				g_critical("%s: setsockopt(%s, IPPROTO_TCP, TCP_NODELAY) failed: %s (%d)", 
+						G_STRLOC,
+						con->dst->name->str,
+						g_strerror(errno), errno);
+				return NETWORK_SOCKET_ERROR;
+			}
+			
+			if (0 != setsockopt(con->fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
+				g_critical("%s: setsockopt(%s, SOL_SOCKET, SO_REUSEADDR) failed: %s (%d)", 
+						G_STRLOC,
+						con->dst->name->str,
+						g_strerror(errno), errno);
+				return NETWORK_SOCKET_ERROR;
+			}
+		}
+
+		if (-1 == bind(con->fd, &con->dst->addr.common, con->dst->len)) {
+			g_critical("%s: bind(%s) failed: %s (%d)", 
 					G_STRLOC,
 					con->dst->name->str,
 					g_strerror(errno), errno);
 			return NETWORK_SOCKET_ERROR;
 		}
-	}
 
-	if (-1 == bind(con->fd, &con->dst->addr.common, con->dst->len)) {
-		g_critical("%s: bind(%s) failed: %s (%d)", 
-				G_STRLOC,
-				con->dst->name->str,
-				g_strerror(errno), errno);
-		return NETWORK_SOCKET_ERROR;
-	}
+		if (-1 == listen(con->fd, 128)) {
+			g_critical("%s: listen(%s, 128) failed: %s (%d)",
+					G_STRLOC,
+					con->dst->name->str,
+					g_strerror(errno), errno);
+			return NETWORK_SOCKET_ERROR;
+		}
+	} else {
+		/* UDP sockets bind the ->src address */
+		g_return_val_if_fail(con->src, NETWORK_SOCKET_ERROR);
+		g_return_val_if_fail(con->src->name->len > 0, NETWORK_SOCKET_ERROR);
 
-	if (-1 == listen(con->fd, 8)) {
-		g_critical("%s: listen(%s, 8) failed: %s (%d)",
-				G_STRLOC,
-				con->dst->name->str,
-				g_strerror(errno), errno);
-		return NETWORK_SOCKET_ERROR;
+		if (-1 == (con->fd = socket(con->src->addr.common.sa_family, con->socket_type, 0))) {
+			g_critical("%s: socket(%s) failed: %s (%d)", 
+					G_STRLOC,
+					con->src->name->str,
+					g_strerror(errno), errno);
+			return NETWORK_SOCKET_ERROR;
+		}
+
+		if (-1 == bind(con->fd, &con->src->addr.common, con->src->len)) {
+			g_critical("%s: bind(%s) failed: %s (%d)", 
+					G_STRLOC,
+					con->src->name->str,
+					g_strerror(errno), errno);
+			return NETWORK_SOCKET_ERROR;
+		}
 	}
 
 	return NETWORK_SOCKET_SUCCESS;
@@ -529,7 +443,15 @@ network_socket_retval_t network_socket_read(network_socket *sock) {
 
 		g_queue_push_tail(sock->recv_queue_raw->chunks, packet);
 
-		if (-1 == (len = recv(sock->fd, packet->str, sock->to_read, 0))) {
+		if (sock->socket_type == SOCK_STREAM) {
+			len = recv(sock->fd, packet->str, sock->to_read, 0);
+		} else {
+			/* UDP */
+			socklen_t dst_len = sizeof(sock->dst->addr.common);
+			len = recvfrom(sock->fd, packet->str, sock->to_read, 0, &(sock->dst->addr.common), &(dst_len));
+			sock->dst->len = dst_len;
+		}
+		if (-1 == len) {
 #ifdef _WIN32
 			errno = WSAGetLastError();
 #endif
@@ -673,7 +595,7 @@ static network_socket_retval_t network_socket_write_writev(network_socket *con,
  *
  * use a loop over send() to be compatible with win32
  */
-static network_socket_retval_t G_GNUC_UNUSED network_socket_write_send(network_socket *con, int send_chunks) {
+static network_socket_retval_t network_socket_write_send(network_socket *con, int send_chunks) {
 	/* send the whole queue */
 	GList *chunk;
 
@@ -685,7 +607,12 @@ static network_socket_retval_t G_GNUC_UNUSED network_socket_write_send(network_s
 
 		g_assert(con->send_queue->offset < s->len);
 
-		if (-1 == (len = send(con->fd, s->str + con->send_queue->offset, s->len - con->send_queue->offset, 0))) {
+		if (con->socket_type == SOCK_STREAM) {
+			len = send(con->fd, s->str + con->send_queue->offset, s->len - con->send_queue->offset, 0);
+		} else {
+			len = sendto(con->fd, s->str + con->send_queue->offset, s->len - con->send_queue->offset, 0, &(con->dst->addr.common), con->dst->len);
+		}
+		if (-1 == len) {
 #ifdef _WIN32
 			errno = WSAGetLastError();
 #endif
@@ -732,21 +659,51 @@ static network_socket_retval_t G_GNUC_UNUSED network_socket_write_send(network_s
 /**
  * write a content of con->send_queue to the socket
  *
+ * @param con         socket to read from
  * @param send_chunks number of chunks to send, if < 0 send all
  *
  * @returns NETWORK_SOCKET_SUCCESS on success, NETWORK_SOCKET_ERROR on error and NETWORK_SOCKET_WAIT_FOR_EVENT if the call would have blocked 
  */
 network_socket_retval_t network_socket_write(network_socket *con, int send_chunks) {
+	if (con->socket_type == SOCK_STREAM) {
 #ifdef HAVE_WRITEV
-	return network_socket_write_writev(con, send_chunks);
+		return network_socket_write_writev(con, send_chunks);
 #else
-	return network_socket_write_send(con, send_chunks);
+		return network_socket_write_send(con, send_chunks);
 #endif
+	} else {
+		return network_socket_write_send(con, send_chunks);
+	}
 }
 
 network_socket_retval_t network_socket_to_read(network_socket *sock) {
 	int b = -1;
 
+#ifdef SO_NREAD
+	/* on MacOS X ioctl(..., FIONREAD) returns _more_ than what we have in the queue */
+	if (sock->socket_type == SOCK_DGRAM) {
+		socklen_t b_len = sizeof(b);
+
+		if (0 != getsockopt(sock->fd, SOL_SOCKET, SO_NREAD, &b, &b_len)) {
+			g_critical("%s: getsockopt(%d, SO_NREAD, ...) failed: %s (%d)",
+					G_STRLOC,
+					sock->fd,
+					g_strerror(errno), errno);
+			return NETWORK_SOCKET_ERROR;
+		} else if (b < 0) {
+			g_critical("%s: getsockopt(%d, SO_NREAD, ...) succeeded, but is negative: %d",
+					G_STRLOC,
+					sock->fd,
+					b);
+
+			return NETWORK_SOCKET_ERROR;
+		} else {
+			sock->to_read = b;
+			return NETWORK_SOCKET_SUCCESS;
+		}
+	}
+#endif
+
 	if (0 != ioctl(sock->fd, FIONREAD, &b)) {
 		g_critical("%s: ioctl(%d, FIONREAD, ...) failed: %s (%d)",
 				G_STRLOC,
