@@ -31,6 +31,21 @@ tests_to_skip = {
         ['select_affected_rows']   = 'needs backends',
         ['client_address_socket'] = 'waiting for bug#38416',
         ['change_user'] = 'works, but needs to run as root and configured with a valid user',
+	['bug_45167'] = 'works, but mysqltest cant handle errors in change_user'
 }
 
+local build_os = os.getenv("BUILD_OS")
+
+--
+-- some older OSes run out of memory in the 32M byte test on our build
+-- platforms. We disable just that test on those platforms
+if build_os and
+	(build_os == "i386-pc-solaris2.8" or
+	 build_os == "x86_64-pc-solaris2.8" or
+	 build_os == "sparc-sun-solaris2.8" or
+	 build_os == "sparc-sun-solaris2.9" or
+	 build_os == "powerpc-ibm-aix5.3.0.0") then
+	tests_to_skip['overlong'] = "can't allocate more than 32M"
+end
+
 
