@@ -29,6 +29,7 @@
 
 #if GLIB_CHECK_VERSION(2, 16, 0)
 #define C(x) x, sizeof(x) - 1
+#define S(x) x->str, x->len
 
 /**
  * Tests for the MySQL Protocol Codec functions
@@ -43,11 +44,18 @@
  * how to handle > 16M ?
  */
 void test_mysqld_proto_header(void) {
+	GString h;
 	unsigned char header[4];
-	size_t length = 1256;
+	guint32 length = 1256;
+	guint8 id = 25;
 
-	g_assert(0 == network_mysqld_proto_set_header(header, length, 0));
-	g_assert(length == network_mysqld_proto_get_header(header));
+	h.str = (char *)header;
+	h.len = sizeof(header);
+
+	g_assert(0 == network_mysqld_proto_set_packet_len(&h, length));
+	g_assert(0 == network_mysqld_proto_set_packet_id(&h, id));
+	g_assert(length == network_mysqld_proto_get_packet_len(&h));
+	g_assert(id == network_mysqld_proto_get_packet_id(&h));
 }
 
 /**
@@ -128,7 +136,7 @@ void test_mysqld_proto_int(void) {
 	g_string_truncate(packet.data, 0);
 	g_assert(0 == network_mysqld_proto_append_int8(packet.data, length));
 	g_assert(packet.data->len == 1);
-	g_assert(0 == memcmp(packet.data->str, C("\xfa")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xfa")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int8(&packet, &value8));
 	g_assert_cmpint(length, ==, value8);
 
@@ -136,7 +144,7 @@ void test_mysqld_proto_int(void) {
 	g_string_truncate(packet.data, 0);
 	g_assert(0 == network_mysqld_proto_append_int16(packet.data, length));
 	g_assert(packet.data->len == 2);
-	g_assert(0 == memcmp(packet.data->str, C("\xfa\xff")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xfa\xff")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int16(&packet, &value16));
 	g_assert_cmpint(length, ==, value16);
 
@@ -144,7 +152,7 @@ void test_mysqld_proto_int(void) {
 	g_string_truncate(packet.data, 0);
 	g_assert(0 == network_mysqld_proto_append_int24(packet.data, length));
 	g_assert(packet.data->len == 3);
-	g_assert(0 == memcmp(packet.data->str, C("\xfa\xff\xff")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xfa\xff\xff")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int24(&packet, &value32));
 	g_assert_cmpint(length, ==, value32);
 
@@ -152,7 +160,7 @@ void test_mysqld_proto_int(void) {
 	g_string_truncate(packet.data, 0);
 	g_assert(0 == network_mysqld_proto_append_int32(packet.data, length));
 	g_assert(packet.data->len == 4);
-	g_assert(0 == memcmp(packet.data->str, C("\xfa\xff\xff\xff")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xfa\xff\xff\xff")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int32(&packet, &value32));
 	g_assert_cmpint(length, ==, value32);
 
@@ -160,7 +168,7 @@ void test_mysqld_proto_int(void) {
 	g_string_truncate(packet.data, 0);
 	g_assert(0 == network_mysqld_proto_append_int48(packet.data, length));
 	g_assert(packet.data->len == 6);
-	g_assert(0 == memcmp(packet.data->str, C("\xfa\xff\xff\xff\x00\x00")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xfa\xff\xff\xff\x00\x00")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int48(&packet, &value64));
 	g_assert_cmpint(length, ==, value64);
 
@@ -171,7 +179,7 @@ void test_mysqld_proto_int(void) {
 	g_string_truncate(packet.data, 0);
 	g_assert(0 == network_mysqld_proto_append_int64(packet.data, length));
 	g_assert(packet.data->len == 8);
-	g_assert(0 == memcmp(packet.data->str, C("\xfa\xff\xff\xff\xff\x00\x00\x00")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xfa\xff\xff\xff\xff\x00\x00\x00")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int64(&packet, &value64));
 	g_assert_cmpuint(length, ==, value64);
 
@@ -181,12 +189,12 @@ void test_mysqld_proto_int(void) {
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_int8(packet.data, 0xa5));
 	g_assert_cmpint(packet.data->len, ==, 1);
 	g_assert_cmpint(packet.offset, ==, 0);
-	g_assert_cmpint(0, ==, memcmp(packet.data->str, C("\xa5")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xa5")));
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_int8(packet.data, 0x5a));
 	g_assert_cmpint(packet.data->len, ==, 2);
 	g_assert_cmpint(packet.offset, ==, 0);
-	g_assert_cmpint(0, ==, memcmp(packet.data->str, C("\xa5\x5a")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), C("\xa5\x5a")));
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_int8(&packet, &value8));
 	g_assert_cmpint(packet.offset, ==, 1);
 	g_assert_cmpint(0xa5, ==, value8);
@@ -371,6 +379,65 @@ void test_mysqld_proto_gstring_len(void) {
 	g_string_free(packet.data, TRUE);
 }
 
+void test_mysqld_proto_gstring(void) {
+	network_packet packet;
+	GString *value = g_string_new(NULL);
+
+	packet.data = g_string_new(NULL);
+
+	packet.offset = 0;
+	g_string_truncate(packet.data, 0);
+	g_assert_cmpint(0, !=, network_mysqld_proto_get_gstring(&packet, value));
+	g_assert_cmpint(0, ==, value->len);
+
+	packet.offset = 0;
+	g_string_assign_len(packet.data, C("012345")); /* no trailing \0 */
+	g_assert_cmpint(0, !=, network_mysqld_proto_get_gstring(&packet, value));
+
+	packet.offset = 0;
+	g_string_assign_len(packet.data, C("012345\0"));
+	g_assert_cmpint(0, ==, network_mysqld_proto_get_gstring(&packet, value));
+	g_assert_cmpint(6, ==, value->len);
+	g_assert_cmpstr("012345", ==, value->str);
+
+	g_string_free(value, TRUE);
+	g_string_free(packet.data, TRUE);
+}
+
+
+void test_mysqld_password(void) {
+	GString *cleartext = g_string_new("123");
+	GString *hashed_password = g_string_new(NULL);
+	GString *double_hashed = g_string_new(NULL);
+	GString *challenge = g_string_new(NULL);
+	GString *response = g_string_new(NULL);
+
+	network_mysqld_proto_password_hash(hashed_password, S(cleartext));
+	network_mysqld_proto_password_hash(double_hashed, S(hashed_password));
+
+	/* should be the same as SELECT SHA1("123"); */
+	g_assert_cmpint(TRUE, ==, g_memeq(S(hashed_password), C("\x40\xbd\x00\x15\x63\x08\x5f\xc3\x51\x65\x32\x9e\xa1\xff\x5c\x5e\xcb\xdb\xbe\xef")));
+	/* should be the same as SELECT PASSWORD("123"); */
+	g_assert_cmpint(TRUE, ==, g_memeq(S(double_hashed), C("\x23\xAE\x80\x9D\xDA\xCA\xF9\x6A\xF0\xFD\x78\xED\x04\xB6\xA2\x65\xE0\x5A\xA2\x57")));
+
+	g_string_assign_len(challenge, C("01234567890123456789"));
+
+	network_mysqld_proto_password_scramble(response, 
+			S(challenge),
+			S(hashed_password));
+	
+	g_assert_cmpint(TRUE, ==, network_mysqld_proto_password_check(
+			S(challenge),
+			S(response),
+			S(double_hashed)));
+
+	g_string_free(response, TRUE);
+	g_string_free(challenge, TRUE);
+	g_string_free(hashed_password, TRUE);
+	g_string_free(double_hashed, TRUE);
+	g_string_free(cleartext, TRUE);
+}
+
 int main(int argc, char **argv) {
 	g_test_init(&argc, &argv, NULL);
 	g_test_bug_base("http://bugs.mysql.com/");
@@ -379,8 +446,10 @@ int main(int argc, char **argv) {
 	g_test_add_func("/core/mysqld-proto-lenenc-int", test_mysqld_proto_lenenc_int);
 	g_test_add_func("/core/mysqld-proto-int", test_mysqld_proto_int);
 	g_test_add_func("/core/mysqld-proto-gstring-len", test_mysqld_proto_gstring_len);
+	g_test_add_func("/core/mysqld-proto-gstring", test_mysqld_proto_gstring);
 
 	g_test_add_func("/core/mysqld-proto-binlog-event", test_mysqld_binlog_events);
+	g_test_add_func("/core/mysqld-proto-password", test_mysqld_password);
 
 	return g_test_run();
 }
