@@ -57,7 +57,7 @@ void t_ok_packet_append(void) {
 	/* check if a empty ok-packet is encoded correctly */
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_ok_packet(packet->data, ok_packet));
 	g_assert_cmpint(7, ==, packet->data->len);
-	g_assert_cmpint(0, ==, memcmp(packet->data->str, C("\x00\x00\x00\x00\x00\x00\x00")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet->data), C("\x00\x00\x00\x00\x00\x00\x00")));
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_ok_packet(packet, ok_packet));
 
@@ -72,7 +72,7 @@ void t_ok_packet_append(void) {
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_ok_packet(packet->data, ok_packet));
 	g_assert_cmpint(7, ==, packet->data->len);
-	g_assert_cmpint(0, ==, memcmp(packet->data->str, C("\x00\x04\x03\x02\x00\x01\x00")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet->data), C("\x00\x04\x03\x02\x00\x01\x00")));
 	
 	network_mysqld_ok_packet_free(ok_packet);
 
@@ -117,7 +117,7 @@ void t_err_packet_append(void) {
 	/* check if a empty ok-packet is encoded correctly */
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_err_packet(packet->data, err_packet));
 	g_assert_cmpint(9, ==, packet->data->len);
-	g_assert_cmpint(0, ==, memcmp(packet->data->str, C("\xff\x00\x00#07000")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet->data), C("\xff\x00\x00#07000")));
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_err_packet(packet, err_packet));
 
@@ -131,7 +131,7 @@ void t_err_packet_append(void) {
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_err_packet(packet->data, err_packet));
 	g_assert_cmpint(13, ==, packet->data->len);
-	g_assert_cmpint(0, ==, memcmp(packet->data->str, C("\xff\x03\x00#01234test")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet->data), C("\xff\x03\x00#01234test")));
 	
 	network_mysqld_err_packet_free(err_packet);
 
@@ -175,7 +175,7 @@ void t_eof_packet_append(void) {
 	/* check if a empty ok-packet is encoded correctly */
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_eof_packet(packet->data, eof_packet));
 	g_assert_cmpint(5, ==, packet->data->len);
-	g_assert_cmpint(0, ==, memcmp(packet->data->str, C("\xfe\x00\x00\x00\x00")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet->data), C("\xfe\x00\x00\x00\x00")));
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_get_eof_packet(packet, eof_packet));
 
@@ -188,7 +188,7 @@ void t_eof_packet_append(void) {
 
 	g_assert_cmpint(0, ==, network_mysqld_proto_append_eof_packet(packet->data, eof_packet));
 	g_assert_cmpint(5, ==, packet->data->len);
-	g_assert_cmpint(0, ==, memcmp(packet->data->str, C("\xfe\x01\x00\x02\x00")));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet->data), C("\xfe\x01\x00\x02\x00")));
 	
 	network_mysqld_eof_packet_free(eof_packet);
 
@@ -334,7 +334,7 @@ void test_mysqld_auth_with_pw(void) {
 			"%@R[SoWC"      /* part 1 */
 			"+L|LG_+R={tV"; /* part 2 */
 
-	GString *packet, *challenge;
+	GString *packet, *challenge, *hashed_password;
 	network_mysqld_auth_response *auth;
 
 	auth = network_mysqld_auth_response_new();
@@ -353,9 +353,13 @@ void test_mysqld_auth_with_pw(void) {
 	auth->charset         = 8;
 
 	challenge = g_string_new(NULL);
+	hashed_password = g_string_new(NULL);
 	g_string_append_len(challenge, raw_challenge, sizeof(raw_challenge) - 1);
 
-	network_mysqld_proto_scramble(auth->response, challenge, "123");
+	network_mysqld_proto_password_hash(hashed_password, C("123"));
+	network_mysqld_proto_password_scramble(auth->response, S(challenge), S(hashed_password));
+
+	g_string_free(hashed_password, TRUE);
 	
 	packet = g_string_new(NULL);
 
@@ -381,14 +385,26 @@ void test_mysqld_auth_with_pw(void) {
 	g_string_free(challenge, TRUE);
 }
 
+/**
+ * @test
+ *   network_mysqld_auth_response_new() and network_mysqld_auth_response_free() 
+ *   don't cause a crash 
+ */
 void t_auth_response_new() {
 	network_mysqld_auth_response *shake;
 
 	shake = network_mysqld_auth_response_new();
+	g_assert(shake);
 
 	network_mysqld_auth_response_free(shake);
 }
 
+/**
+ * @test
+ *   network_mysqld_proto_get_auth_response() can decode a string
+ *   network_mysqld_proto_append_auth_response() can encode the result
+ *     of the network_mysqld_proto_get_auth_response() 
+ */
 void t_mysqld_get_auth_response(void) {
 	const char raw_packet[] = 
 		"\205\246\3\0"
@@ -435,7 +451,7 @@ void t_mysqld_get_auth_response(void) {
 	g_assert_cmpint(err, ==, 0);
 
 	g_assert_cmpint(packet.data->len, ==, sizeof(raw_packet) - 1);
-	g_assert_cmpint(0, ==, memcmp(packet.data->str, raw_packet, packet.data->len));
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), raw_packet, packet.data->len));
 
 	network_mysqld_auth_response_free(auth);
 
@@ -451,8 +467,237 @@ void t_mysqld_get_auth_response(void) {
 	g_string_free(packet.data, TRUE);
 }
 
+/**
+ * @test
+ *   network_mysqld_proto_get_auth_response() can decode a pre-4.0 packet
+ *   network_mysqld_proto_append_auth_response() can encode the result
+ *     of the network_mysqld_proto_get_auth_response() 
+ */
+void t_mysqld_get_auth_response_pre_41(void) {
+	const char raw_packet[] = 
+		"\205$"
+		"\0\0\0"
+		"root\0"
+		;
+
+	network_mysqld_auth_response *auth;
+	network_packet packet;
+	int err = 0;
+
+	auth = network_mysqld_auth_response_new();
+	packet.data = g_string_new_len(C(raw_packet));
+	packet.offset = 0;
+
+	err = err || network_mysqld_proto_get_auth_response(&packet, auth);
+
+	g_assert_cmpint(err, ==, 0);
+
+	g_assert(auth->username);
+	g_assert_cmpint(auth->username->len, ==, 4);
+	g_assert_cmpstr(auth->username->str, ==, "root");
+
+	g_assert_cmpuint(auth->capabilities, ==,
+		CLIENT_LONG_PASSWORD |
+	       	CLIENT_LONG_FLAG |
+		CLIENT_LOCAL_FILES | 
+		CLIENT_INTERACTIVE |
+		CLIENT_TRANSACTIONS 
+		); 
+	g_assert_cmpuint(auth->max_packet_size, ==, 0);
+
+	g_string_truncate(packet.data, 0);
+	packet.offset = 0;
+
+	err = err || network_mysqld_proto_append_auth_response(packet.data, auth);
+	g_assert_cmpint(err, ==, 0);
+
+	g_assert_cmpint(packet.data->len, ==, sizeof(raw_packet) - 1);
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), raw_packet, packet.data->len));
+
+	network_mysqld_auth_response_free(auth);
+
+	/* empty auth struct */
+	g_string_truncate(packet.data, 0);
+	packet.offset = 0;
+
+	auth = network_mysqld_auth_response_new();
+	err = err || network_mysqld_proto_append_auth_response(packet.data, auth);
+	g_assert_cmpint(err, ==, 0);
+	network_mysqld_auth_response_free(auth);
+
+	g_string_free(packet.data, TRUE);
+}
+
+/**
+ * @test
+ *   network_mysqld_proto_get_auth_response() can decode a broken pre-4.0 packet
+ */
+void t_mysqld_get_auth_response_no_term(void) {
+	const char raw_packet[] = 
+		"\205$"
+		"\0\0\0"
+		"root\0" /* missing trailing \0 */
+		"foo"
+		;
+
+	network_mysqld_auth_response *auth;
+	network_packet packet;
+	int err = 0;
+
+	auth = network_mysqld_auth_response_new();
+	packet.data = g_string_new_len(C(raw_packet));
+	packet.offset = 0;
+
+	err = err || network_mysqld_proto_get_auth_response(&packet, auth);
+
+	g_assert_cmpint(err, !=, 0);
+
+	network_mysqld_auth_response_free(auth);
+
+	g_string_free(packet.data, TRUE);
+}
 
 
+typedef struct {
+	const char *s;
+	size_t s_len;
+} strings;
+
+void t_resultset_fields_works(void) {
+	strings packets[] = {
+		{ C("\1\0\0\1\2") }, /* 2 fields */
+		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
+		{ C("&\0\0\3\3def\0\6STATUS\0\5Value\5Value\f\10\0\0\2\0\0\375\1\0\0\0\0") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ C("\23\0\0\5\17Aborted_clients\00298") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ NULL, 0 }
+	};
+	int i;
+	network_queue *q;
+	GPtrArray *fields;
+
+	q = network_queue_new();
+
+	for (i = 0; packets[i].s; i++) {
+		network_queue_append(q, g_string_new_len(packets[i].s, packets[i].s_len));
+	}
+
+	fields = g_ptr_array_new();
+	g_assert(NULL != network_mysqld_proto_get_fielddefs(q->chunks->head, fields));
+
+	network_queue_free(q);
+}
+
+void t_resultset_fields_parse_err(void) {
+	strings packets[] = {
+		{ C("\1\0\0\1\377") }, /* err-packet */
+		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
+		{ C("&\0\0\3\3def\0\6STATUS\0\5Value\5Value\f\10\0\0\2\0\0\375\1\0\0\0\0") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ C("\23\0\0\5\17Aborted_clients\00298") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ NULL, 0 }
+	};
+	int i;
+	network_queue *q;
+	GPtrArray *fields;
+
+	q = network_queue_new();
+
+	for (i = 0; packets[i].s; i++) {
+		network_queue_append(q, g_string_new_len(packets[i].s, packets[i].s_len));
+	}
+
+	fields = g_ptr_array_new();
+	g_assert(NULL == network_mysqld_proto_get_fielddefs(q->chunks->head, fields));
+
+	network_queue_free(q);
+}
+
+void t_resultset_fields_parse_null(void) {
+	strings packets[] = {
+		{ C("\1\0\0\1\373") }, /* NULL */
+		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
+		{ C("&\0\0\3\3def\0\6STATUS\0\5Value\5Value\f\10\0\0\2\0\0\375\1\0\0\0\0") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ C("\23\0\0\5\17Aborted_clients\00298") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ NULL, 0 }
+	};
+	int i;
+	network_queue *q;
+	GPtrArray *fields;
+
+	q = network_queue_new();
+
+	for (i = 0; packets[i].s; i++) {
+		network_queue_append(q, g_string_new_len(packets[i].s, packets[i].s_len));
+	}
+
+	fields = g_ptr_array_new();
+	g_assert(NULL == network_mysqld_proto_get_fielddefs(q->chunks->head, fields));
+
+	network_queue_free(q);
+}
+
+void t_resultset_fields_parse_low(void) {
+	strings packets[] = {
+		{ C("\1\0\0\1\2") }, /* 2 fields */
+		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ C("\23\0\0\5\17Aborted_clients\00298") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ NULL, 0 }
+	};
+	int i;
+	network_queue *q;
+	GPtrArray *fields;
+
+	q = network_queue_new();
+
+	for (i = 0; packets[i].s; i++) {
+		network_queue_append(q, g_string_new_len(packets[i].s, packets[i].s_len));
+	}
+
+	fields = g_ptr_array_new();
+	g_assert(NULL == network_mysqld_proto_get_fielddefs(q->chunks->head, fields));
+
+	network_queue_free(q);
+}
+
+void t_resultset_fields_parse_high(void) {
+	strings packets[] = {
+		{ C("\1\0\0\1\2") }, /* 2 fields */
+		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
+		{ C("&\0\0\3\3def\0\6STATUS\0\5Value\5Value\f\10\0\0\2\0\0\375\1\0\0\0\0") },
+		{ C("&\0\0\3\3def\0\6STATUS\0\5Value\5Value\f\10\0\0\2\0\0\375\1\0\0\0\0") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ C("\23\0\0\5\17Aborted_clients\00298") },
+		{ C("\5\0\0\4\376\0\0\"\0") }, /* EOF */
+		{ NULL, 0 }
+	};
+	int i;
+	network_queue *q;
+	GPtrArray *fields;
+
+	q = network_queue_new();
+
+	for (i = 0; packets[i].s; i++) {
+		network_queue_append(q, g_string_new_len(packets[i].s, packets[i].s_len));
+	}
+
+	fields = g_ptr_array_new();
+	g_assert(NULL == network_mysqld_proto_get_fielddefs(q->chunks->head, fields));
+
+	network_queue_free(q);
+}
+
+
+/**
+ * @cond
+ *   don't include the main() function the docs
+ */
 int main(int argc, char **argv) {
 	g_test_init(&argc, &argv, NULL);
 	g_test_bug_base("http://bugs.mysql.com/");
@@ -471,9 +716,18 @@ int main(int argc, char **argv) {
 	
 	g_test_add_func("/core/mysqld-proto-auth-response-new", t_auth_response_new);
 	g_test_add_func("/core/mysqld-proto-get-auth-response", t_mysqld_get_auth_response);
+	g_test_add_func("/core/mysqld-proto-get-auth-response-pre-4.1", t_mysqld_get_auth_response_pre_41);
+	g_test_add_func("/core/mysqld-proto-get-auth-response-no-term", t_mysqld_get_auth_response_no_term);
+	
+	g_test_add_func("/core/resultset-fields", t_resultset_fields_works);
+	g_test_add_func("/core/resultset-fields-broken-proto-err", t_resultset_fields_parse_err);
+	g_test_add_func("/core/resultset-fields-broken-proto-null", t_resultset_fields_parse_null);
+	g_test_add_func("/core/resultset-fields-broken-proto-field-count-low", t_resultset_fields_parse_low);
+	g_test_add_func("/core/resultset-fields-broken-proto-field-count-high", t_resultset_fields_parse_high);
 
 	return g_test_run();
 }
+/** @endcond */
 #else
 int main() {
 	return 77;
